# Test MSOffice
#
# Main purpose of test is to ensure that Dynamic COM objects
# work as expected.

# Assumes Word and Excel installed on your machine.

import win32com, sys, string, win32api, traceback
import win32com.client.dynamic
from win32com.test.util import CheckClean
import pythoncom
from win32com.client import gencache
from pywintypes import Unicode

error = "MSOffice test error"

# Test a few of the MSOffice components.
def TestWord():
    # Try and load the object exposed by Word 8
    # Office 97 - _totally_ different object model!
    try:
        # NOTE - using "client.Dispatch" would return an msword8.py instance!
        print("Starting Word 8 for dynamic test")
        word = win32com.client.dynamic.Dispatch("Word.Application")
        TestWord8(word)

        word = None
        # Now we will test Dispatch without the new "lazy" capabilities
        print("Starting Word 8 for non-lazy dynamic test")
        dispatch = win32com.client.dynamic._GetGoodDispatch("Word.Application")
        typeinfo = dispatch.GetTypeInfo()
        attr = typeinfo.GetTypeAttr()
        olerepr = win32com.client.build.DispatchItem(typeinfo, attr, None, 0)
        word = win32com.client.dynamic.CDispatch(dispatch, olerepr)
        dispatch = typeinfo = attr = olerepr = None
        TestWord8(word)

    except pythoncom.com_error:
        print("Starting Word 7 for dynamic test")
        word = win32com.client.Dispatch("Word.Basic")
        TestWord7(word)

    except Exception as e:
        print("Word dynamic tests failed", e)
        traceback.print_exc()

    print("Starting MSWord for generated test")
    try:
        from win32com.client import gencache

        word = gencache.EnsureDispatch("Word.Application.8")
        TestWord8(word)
    except Exception as e:
        print("Word generated tests failed", e)
        traceback.print_exc()


def TestWord7(word):
    word.FileNew()
    # If not shown, show the app.
    if not word.AppShow():
        word._proc_("AppShow")

    for i in range(12):
        word.FormatFont(Color=i + 1, Points=i + 12)
        word.Insert("Hello from Python %d\n" % i)

    word.FileClose(2)


def TestWord8(word):
    word.Visible = 1
    doc = word.Documents.Add()
    wrange = doc.Range()
    for i in range(10):
        wrange.InsertAfter("Hello from Python %d\n" % i)
    paras = doc.Paragraphs
    for i in range(len(paras)):
        # *sob* - in Word 2019, `p = paras(i+1)` seems to work to get a para
        # but `p.Font` then blows up.
        #        p = paras[i]()
        p = paras(i + 1)
        p.Font.ColorIndex = i + 1
        p.Font.Size = 12 + (4 * i)
    # XXX - note that
    # for para in paras:
    #       para().Font...
    # doesnt seem to work - no error, just doesnt work
    # Should check if it works for VB!
    doc.Close(SaveChanges=0)
    word.Quit()
    win32api.Sleep(1000)  # Wait for word to close, else we
    # may get OA error.


def TestWord8OldStyle():
    try:
        import win32com.test.Generated4Test.msword8
    except ImportError:
        print("Can not do old style test")


def TextExcel(xl):
    xl.Visible = 0
    if xl.Visible:
        raise error("Visible property is true.")
    xl.Visible = 1
    if not xl.Visible:
        raise error("Visible property not true.")

    if int(xl.Version[0]) >= 8:
        xl.Workbooks.Add()
    else:
        xl.Workbooks().Add()

    xl.Range("A1:C1").Value = (1, 2, 3)
    xl.Range("A2:C2").Value = ("x", "y", "z")
    xl.Range("A3:C3").Value = ("3", "2", "1")

    for i in range(20):
        xl.Cells(i + 1, i + 1).Value = "Hi %d" % i

    if xl.Range("A1").Value != "Hi 0":
        raise error("Single cell range failed")

    if xl.Range("A1:B1").Value != ((Unicode("Hi 0"), 2),):
        raise error("flat-horizontal cell range failed")

    if xl.Range("A1:A2").Value != ((Unicode("Hi 0"),), (Unicode("x"),)):
        raise error("flat-vertical cell range failed")

    if xl.Range("A1:C3").Value != (
        (Unicode("Hi 0"), 2, 3),
        (Unicode("x"), Unicode("Hi 1"), Unicode("z")),
        (3, 2, Unicode("Hi 2")),
    ):
        raise error("square cell range failed")

    xl.Range("A1:C3").Value = ((3, 2, 1), ("x", "y", "z"), (1, 2, 3))

    if xl.Range("A1:C3").Value != (
        (3, 2, 1),
        (Unicode("x"), Unicode("y"), Unicode("z")),
        (1, 2, 3),
    ):
        raise error("Range was not what I set it to!")

    # test dates out with Excel
    xl.Cells(5, 1).Value = "Excel time"
    xl.Cells(5, 2).Formula = "=Now()"

    import time

    xl.Cells(6, 1).Value = "Python time"
    xl.Cells(6, 2).Value = pythoncom.MakeTime(time.time())
    xl.Cells(6, 2).NumberFormat = "d/mm/yy h:mm"
    xl.Columns("A:B").EntireColumn.AutoFit()

    xl.Workbooks(1).Close(0)
    xl.Quit()


def TestAll():
    TestWord()

    try:
        print("Starting Excel for Dynamic test...")
        xl = win32com.client.dynamic.Dispatch("Excel.Application")
        TextExcel(xl)
    except Exception as e:
        worked = False
        print("Excel tests failed", e)
        traceback.print_exc()

    try:
        print("Starting Excel 8 for generated excel8.py test...")
        mod = gencache.EnsureModule(
            "{00020813-0000-0000-C000-000000000046}", 0, 1, 2, bForDemand=1
        )
        xl = win32com.client.Dispatch("Excel.Application")
        TextExcel(xl)
    except ImportError:
        print("Could not import the generated Excel 97 wrapper")
    except Exception as e:
        print("Generated Excel tests failed", e)
        traceback.print_exc()

    try:
        import xl5en32

        mod = gencache.EnsureModule("{00020813-0000-0000-C000-000000000046}", 9, 1, 0)
        xl = win32com.client.Dispatch("Excel.Application.5")
        print("Starting Excel 95 for makepy test...")
        TextExcel(xl)
    except ImportError:
        print("Could not import the generated Excel 95 wrapper")
    except Exception as e:
        print("Excel 95 tests failed", e)
        traceback.print_exc()


if __name__ == "__main__":
    TestAll()
    CheckClean()
    pythoncom.CoUninitialize()



# Converted "manually" from EMSABTAG.H
from .mapitags import (
    PT_UNSPECIFIED,
    PT_NULL,
    PT_I2,
    PT_LONG,
    PT_R4,
    PT_DOUBLE,
    PT_CURRENCY,
    PT_APPTIME,
    PT_ERROR,
    PT_BOOLEAN,
    PT_OBJECT,
    PT_I8,
    PT_STRING8,
    PT_UNICODE,
    PT_SYSTIME,
    PT_CLSID,
    PT_BINARY,
    PT_SHORT,
    PT_I4,
    PT_FLOAT,
    PT_DOUBLE,
    PT_LONGLONG,
    PT_TSTRING,
    PT_MV_I2,
    PT_MV_LONG,
    PT_MV_R4,
    PT_MV_DOUBLE,
    PT_MV_CURRENCY,
    PT_MV_APPTIME,
    PT_MV_SYSTIME,
    PT_MV_STRING8,
    PT_MV_BINARY,
    PT_MV_UNICODE,
    PT_MV_CLSID,
    PT_MV_I8,
    PT_MV_SHORT,
    PT_MV_I4,
    PT_MV_FLOAT,
    PT_MV_R8,
    PT_MV_LONGLONG,
    PT_MV_TSTRING,
    PROP_TAG,
)


AB_SHOW_PHANTOMS = 2
AB_SHOW_OTHERS = 4

# Flags for ulFlag on ResolveNames
EMS_AB_ADDRESS_LOOKUP = 1


# Constructed, but externally visible.
PR_EMS_AB_SERVER = PROP_TAG(PT_TSTRING, 65534)
PR_EMS_AB_SERVER_A = PROP_TAG(PT_STRING8, 65534)
PR_EMS_AB_SERVER_W = PROP_TAG(PT_UNICODE, 65534)
PR_EMS_AB_CONTAINERID = PROP_TAG(PT_LONG, 65533)
PR_EMS_AB_DOS_ENTRYID = PR_EMS_AB_CONTAINERID
PR_EMS_AB_PARENT_ENTRYID = PROP_TAG(PT_BINARY, 65532)
PR_EMS_AB_IS_MASTER = PROP_TAG(PT_BOOLEAN, 65531)
PR_EMS_AB_OBJECT_OID = PROP_TAG(PT_BINARY, 65530)
PR_EMS_AB_HIERARCHY_PATH = PROP_TAG(PT_TSTRING, 65529)
PR_EMS_AB_HIERARCHY_PATH_A = PROP_TAG(PT_STRING8, 65529)
PR_EMS_AB_HIERARCHY_PATH_W = PROP_TAG(PT_UNICODE, 65529)
PR_EMS_AB_CHILD_RDNS = PROP_TAG(PT_MV_STRING8, 65528)

MIN_EMS_AB_CONSTRUCTED_PROP_ID = 65528

PR_EMS_AB_OTHER_RECIPS = PROP_TAG(PT_OBJECT, 61440)

# Prop tags defined in the schema.
PR_EMS_AB_DISPLAY_NAME_PRINTABLE = PROP_TAG(PT_TSTRING, 14847)
PR_EMS_AB_DISPLAY_NAME_PRINTABLE_A = PROP_TAG(PT_STRING8, 14847)
PR_EMS_AB_DISPLAY_NAME_PRINTABLE_W = PROP_TAG(PT_UNICODE, 14847)

PR_EMS_AB_ACCESS_CATEGORY = PROP_TAG(PT_LONG, 32836)
PR_EMS_AB_ACTIVATION_SCHEDULE = PROP_TAG(PT_BINARY, 32837)
PR_EMS_AB_ACTIVATION_STYLE = PROP_TAG(PT_LONG, 32838)
PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE = PROP_TAG(PT_BINARY, 32791)
PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_MSDOS = PROP_TAG(PT_BINARY, 32839)
PR_EMS_AB_ADDRESS_SYNTAX = PROP_TAG(PT_BINARY, 32792)
PR_EMS_AB_ADDRESS_TYPE = PROP_TAG(PT_TSTRING, 32840)
PR_EMS_AB_ADDRESS_TYPE_A = PROP_TAG(PT_STRING8, 32840)
PR_EMS_AB_ADDRESS_TYPE_W = PROP_TAG(PT_UNICODE, 32840)
PR_EMS_AB_ADMD = PROP_TAG(PT_TSTRING, 32841)
PR_EMS_AB_ADMD_A = PROP_TAG(PT_STRING8, 32841)
PR_EMS_AB_ADMD_W = PROP_TAG(PT_UNICODE, 32841)
PR_EMS_AB_ADMIN_DESCRIPTION = PROP_TAG(PT_TSTRING, 32842)
PR_EMS_AB_ADMIN_DESCRIPTION_A = PROP_TAG(PT_STRING8, 32842)
PR_EMS_AB_ADMIN_DESCRIPTION_W = PROP_TAG(PT_UNICODE, 32842)
PR_EMS_AB_ADMIN_DISPLAY_NAME = PROP_TAG(PT_TSTRING, 32843)
PR_EMS_AB_ADMIN_DISPLAY_NAME_A = PROP_TAG(PT_STRING8, 32843)
PR_EMS_AB_ADMIN_DISPLAY_NAME_W = PROP_TAG(PT_UNICODE, 32843)
PR_EMS_AB_ADMIN_EXTENSION_DLL = PROP_TAG(PT_TSTRING, 32844)
PR_EMS_AB_ADMIN_EXTENSION_DLL_A = PROP_TAG(PT_STRING8, 32844)
PR_EMS_AB_ADMIN_EXTENSION_DLL_W = PROP_TAG(PT_UNICODE, 32844)
PR_EMS_AB_ALIASED_OBJECT_NAME = PROP_TAG(PT_TSTRING, 32845)
PR_EMS_AB_ALIASED_OBJECT_NAME_A = PROP_TAG(PT_STRING8, 32845)
PR_EMS_AB_ALIASED_OBJECT_NAME_W = PROP_TAG(PT_UNICODE, 32845)
PR_EMS_AB_ALIASED_OBJECT_NAME_O = PROP_TAG(PT_OBJECT, 32845)
PR_EMS_AB_ALIASED_OBJECT_NAME_T = PROP_TAG(PT_TSTRING, 32845)
PR_EMS_AB_ALT_RECIPIENT = PROP_TAG(PT_TSTRING, 32846)
PR_EMS_AB_ALT_RECIPIENT_A = PROP_TAG(PT_STRING8, 32846)
PR_EMS_AB_ALT_RECIPIENT_W = PROP_TAG(PT_UNICODE, 32846)
PR_EMS_AB_ALT_RECIPIENT_O = PROP_TAG(PT_OBJECT, 32846)
PR_EMS_AB_ALT_RECIPIENT_T = PROP_TAG(PT_TSTRING, 32846)
PR_EMS_AB_ALT_RECIPIENT_BL = PROP_TAG(PT_MV_TSTRING, 32847)
PR_EMS_AB_ALT_RECIPIENT_BL_A = PROP_TAG(PT_MV_STRING8, 32847)
PR_EMS_AB_ALT_RECIPIENT_BL_W = PROP_TAG(PT_MV_UNICODE, 32847)
PR_EMS_AB_ALT_RECIPIENT_BL_O = PROP_TAG(PT_OBJECT, 32847)
PR_EMS_AB_ALT_RECIPIENT_BL_T = PROP_TAG(PT_MV_TSTRING, 32847)
PR_EMS_AB_ANCESTOR_ID = PROP_TAG(PT_BINARY, 32848)
PR_EMS_AB_ASSOC_NT_ACCOUNT = PROP_TAG(PT_BINARY, 32807)
PR_EMS_AB_ASSOC_REMOTE_DXA = PROP_TAG(PT_MV_TSTRING, 32849)
PR_EMS_AB_ASSOC_REMOTE_DXA_A = PROP_TAG(PT_MV_STRING8, 32849)
PR_EMS_AB_ASSOC_REMOTE_DXA_W = PROP_TAG(PT_MV_UNICODE, 32849)
PR_EMS_AB_ASSOC_REMOTE_DXA_O = PROP_TAG(PT_OBJECT, 32849)
PR_EMS_AB_ASSOC_REMOTE_DXA_T = PROP_TAG(PT_MV_TSTRING, 32849)
PR_EMS_AB_ASSOCIATION_LIFETIME = PROP_TAG(PT_LONG, 32850)
PR_EMS_AB_AUTH_ORIG_BL = PROP_TAG(PT_MV_TSTRING, 32851)
PR_EMS_AB_AUTH_ORIG_BL_A = PROP_TAG(PT_MV_STRING8, 32851)
PR_EMS_AB_AUTH_ORIG_BL_W = PROP_TAG(PT_MV_UNICODE, 32851)
PR_EMS_AB_AUTH_ORIG_BL_O = PROP_TAG(PT_OBJECT, 32851)
PR_EMS_AB_AUTH_ORIG_BL_T = PROP_TAG(PT_MV_TSTRING, 32851)
PR_EMS_AB_AUTHORITY_REVOCATION_LIST = PROP_TAG(PT_MV_BINARY, 32806)
PR_EMS_AB_AUTHORIZED_DOMAIN = PROP_TAG(PT_TSTRING, 32852)
PR_EMS_AB_AUTHORIZED_DOMAIN_A = PROP_TAG(PT_STRING8, 32852)
PR_EMS_AB_AUTHORIZED_DOMAIN_W = PROP_TAG(PT_UNICODE, 32852)
PR_EMS_AB_AUTHORIZED_PASSWORD = PROP_TAG(PT_BINARY, 32853)
PR_EMS_AB_AUTHORIZED_USER = PROP_TAG(PT_TSTRING, 32854)
PR_EMS_AB_AUTHORIZED_USER_A = PROP_TAG(PT_STRING8, 32854)
PR_EMS_AB_AUTHORIZED_USER_W = PROP_TAG(PT_UNICODE, 32854)
PR_EMS_AB_AUTOREPLY = PROP_TAG(PT_BOOLEAN, 32779)
PR_EMS_AB_AUTOREPLY_MESSAGE = PROP_TAG(PT_TSTRING, 32778)
PR_EMS_AB_AUTOREPLY_MESSAGE_A = PROP_TAG(PT_STRING8, 32778)
PR_EMS_AB_AUTOREPLY_MESSAGE_W = PROP_TAG(PT_UNICODE, 32778)
PR_EMS_AB_AUTOREPLY_SUBJECT = PROP_TAG(PT_TSTRING, 32830)
PR_EMS_AB_AUTOREPLY_SUBJECT_A = PROP_TAG(PT_STRING8, 32830)
PR_EMS_AB_AUTOREPLY_SUBJECT_W = PROP_TAG(PT_UNICODE, 32830)
PR_EMS_AB_BRIDGEHEAD_SERVERS = PROP_TAG(PT_MV_TSTRING, 33140)
PR_EMS_AB_BRIDGEHEAD_SERVERS_A = PROP_TAG(PT_MV_STRING8, 33140)
PR_EMS_AB_BRIDGEHEAD_SERVERS_W = PROP_TAG(PT_MV_UNICODE, 33140)
PR_EMS_AB_BRIDGEHEAD_SERVERS_O = PROP_TAG(PT_OBJECT, 33140)
PR_EMS_AB_BRIDGEHEAD_SERVERS_T = PROP_TAG(PT_MV_TSTRING, 33140)
PR_EMS_AB_BUSINESS_CATEGORY = PROP_TAG(PT_MV_TSTRING, 32855)
PR_EMS_AB_BUSINESS_CATEGORY_A = PROP_TAG(PT_MV_STRING8, 32855)
PR_EMS_AB_BUSINESS_CATEGORY_W = PROP_TAG(PT_MV_UNICODE, 32855)
PR_EMS_AB_BUSINESS_ROLES = PROP_TAG(PT_BINARY, 32803)
PR_EMS_AB_CA_CERTIFICATE = PROP_TAG(PT_MV_BINARY, 32771)
PR_EMS_AB_CAN_CREATE_PF = PROP_TAG(PT_MV_TSTRING, 32856)
PR_EMS_AB_CAN_CREATE_PF_A = PROP_TAG(PT_MV_STRING8, 32856)
PR_EMS_AB_CAN_CREATE_PF_W = PROP_TAG(PT_MV_UNICODE, 32856)
PR_EMS_AB_CAN_CREATE_PF_O = PROP_TAG(PT_OBJECT, 32856)
PR_EMS_AB_CAN_CREATE_PF_T = PROP_TAG(PT_MV_TSTRING, 32856)
PR_EMS_AB_CAN_CREATE_PF_BL = PROP_TAG(PT_MV_TSTRING, 32857)
PR_EMS_AB_CAN_CREATE_PF_BL_A = PROP_TAG(PT_MV_STRING8, 32857)
PR_EMS_AB_CAN_CREATE_PF_BL_W = PROP_TAG(PT_MV_UNICODE, 32857)
PR_EMS_AB_CAN_CREATE_PF_BL_O = PROP_TAG(PT_OBJECT, 32857)
PR_EMS_AB_CAN_CREATE_PF_BL_T = PROP_TAG(PT_MV_TSTRING, 32857)
PR_EMS_AB_CAN_CREATE_PF_DL = PROP_TAG(PT_MV_TSTRING, 32858)
PR_EMS_AB_CAN_CREATE_PF_DL_A = PROP_TAG(PT_MV_STRING8, 32858)
PR_EMS_AB_CAN_CREATE_PF_DL_W = PROP_TAG(PT_MV_UNICODE, 32858)
PR_EMS_AB_CAN_CREATE_PF_DL_O = PROP_TAG(PT_OBJECT, 32858)
PR_EMS_AB_CAN_CREATE_PF_DL_T = PROP_TAG(PT_MV_TSTRING, 32858)
PR_EMS_AB_CAN_CREATE_PF_DL_BL = PROP_TAG(PT_MV_TSTRING, 32859)
PR_EMS_AB_CAN_CREATE_PF_DL_BL_A = PROP_TAG(PT_MV_STRING8, 32859)
PR_EMS_AB_CAN_CREATE_PF_DL_BL_W = PROP_TAG(PT_MV_UNICODE, 32859)
PR_EMS_AB_CAN_CREATE_PF_DL_BL_O = PROP_TAG(PT_OBJECT, 32859)
PR_EMS_AB_CAN_CREATE_PF_DL_BL_T = PROP_TAG(PT_MV_TSTRING, 32859)
PR_EMS_AB_CAN_NOT_CREATE_PF = PROP_TAG(PT_MV_TSTRING, 32860)
PR_EMS_AB_CAN_NOT_CREATE_PF_A = PROP_TAG(PT_MV_STRING8, 32860)
PR_EMS_AB_CAN_NOT_CREATE_PF_W = PROP_TAG(PT_MV_UNICODE, 32860)
PR_EMS_AB_CAN_NOT_CREATE_PF_O = PROP_TAG(PT_OBJECT, 32860)
PR_EMS_AB_CAN_NOT_CREATE_PF_T = PROP_TAG(PT_MV_TSTRING, 32860)
PR_EMS_AB_CAN_NOT_CREATE_PF_BL = PROP_TAG(PT_MV_TSTRING, 32861)
PR_EMS_AB_CAN_NOT_CREATE_PF_BL_A = PROP_TAG(PT_MV_STRING8, 32861)
PR_EMS_AB_CAN_NOT_CREATE_PF_BL_W = PROP_TAG(PT_MV_UNICODE, 32861)
PR_EMS_AB_CAN_NOT_CREATE_PF_BL_O = PROP_TAG(PT_OBJECT, 32861)
PR_EMS_AB_CAN_NOT_CREATE_PF_BL_T = PROP_TAG(PT_MV_TSTRING, 32861)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL = PROP_TAG(PT_MV_TSTRING, 32862)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_A = PROP_TAG(PT_MV_STRING8, 32862)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_W = PROP_TAG(PT_MV_UNICODE, 32862)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_O = PROP_TAG(PT_OBJECT, 32862)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_T = PROP_TAG(PT_MV_TSTRING, 32862)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL = PROP_TAG(PT_MV_TSTRING, 32863)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_A = PROP_TAG(PT_MV_STRING8, 32863)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_W = PROP_TAG(PT_MV_UNICODE, 32863)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_O = PROP_TAG(PT_OBJECT, 32863)
PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_T = PROP_TAG(PT_MV_TSTRING, 32863)
PR_EMS_AB_CAN_PRESERVE_DNS = PROP_TAG(PT_BOOLEAN, 32864)
PR_EMS_AB_CERTIFICATE_REVOCATION_LIST = PROP_TAG(PT_BINARY, 32790)
PR_EMS_AB_CLOCK_ALERT_OFFSET = PROP_TAG(PT_LONG, 32865)
PR_EMS_AB_CLOCK_ALERT_REPAIR = PROP_TAG(PT_BOOLEAN, 32866)
PR_EMS_AB_CLOCK_WARNING_OFFSET = PROP_TAG(PT_LONG, 32867)
PR_EMS_AB_CLOCK_WARNING_REPAIR = PROP_TAG(PT_BOOLEAN, 32868)
PR_EMS_AB_COMPUTER_NAME = PROP_TAG(PT_TSTRING, 32869)
PR_EMS_AB_COMPUTER_NAME_A = PROP_TAG(PT_STRING8, 32869)
PR_EMS_AB_COMPUTER_NAME_W = PROP_TAG(PT_UNICODE, 32869)
PR_EMS_AB_CONNECTED_DOMAINS = PROP_TAG(PT_MV_TSTRING, 32870)
PR_EMS_AB_CONNECTED_DOMAINS_A = PROP_TAG(PT_MV_STRING8, 32870)
PR_EMS_AB_CONNECTED_DOMAINS_W = PROP_TAG(PT_MV_UNICODE, 32870)
PR_EMS_AB_CONTAINER_INFO = PROP_TAG(PT_LONG, 32871)
PR_EMS_AB_COST = PROP_TAG(PT_LONG, 32872)
PR_EMS_AB_COUNTRY_NAME = PROP_TAG(PT_TSTRING, 32873)
PR_EMS_AB_COUNTRY_NAME_A = PROP_TAG(PT_STRING8, 32873)
PR_EMS_AB_COUNTRY_NAME_W = PROP_TAG(PT_UNICODE, 32873)
PR_EMS_AB_CROSS_CERTIFICATE_PAIR = PROP_TAG(PT_MV_BINARY, 32805)
PR_EMS_AB_DELIV_CONT_LENGTH = PROP_TAG(PT_LONG, 32874)
PR_EMS_AB_DELIV_EITS = PROP_TAG(PT_MV_BINARY, 32875)
PR_EMS_AB_DELIV_EXT_CONT_TYPES = PROP_TAG(PT_MV_BINARY, 32876)
PR_EMS_AB_DELIVER_AND_REDIRECT = PROP_TAG(PT_BOOLEAN, 32877)
PR_EMS_AB_DELIVERY_MECHANISM = PROP_TAG(PT_LONG, 32878)
PR_EMS_AB_DESCRIPTION = PROP_TAG(PT_MV_TSTRING, 32879)
PR_EMS_AB_DESCRIPTION_A = PROP_TAG(PT_MV_STRING8, 32879)
PR_EMS_AB_DESCRIPTION_W = PROP_TAG(PT_MV_UNICODE, 32879)
PR_EMS_AB_DESTINATION_INDICATOR = PROP_TAG(PT_MV_TSTRING, 32880)
PR_EMS_AB_DESTINATION_INDICATOR_A = PROP_TAG(PT_MV_STRING8, 32880)
PR_EMS_AB_DESTINATION_INDICATOR_W = PROP_TAG(PT_MV_UNICODE, 32880)
PR_EMS_AB_DIAGNOSTIC_REG_KEY = PROP_TAG(PT_TSTRING, 32881)
PR_EMS_AB_DIAGNOSTIC_REG_KEY_A = PROP_TAG(PT_STRING8, 32881)
PR_EMS_AB_DIAGNOSTIC_REG_KEY_W = PROP_TAG(PT_UNICODE, 32881)
PR_EMS_AB_DISPLAY_NAME_OVERRIDE = PROP_TAG(PT_BOOLEAN, 32769)
PR_EMS_AB_DL_MEM_REJECT_PERMS_BL = PROP_TAG(PT_MV_TSTRING, 32882)
PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_A = PROP_TAG(PT_MV_STRING8, 32882)
PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_W = PROP_TAG(PT_MV_UNICODE, 32882)
PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_O = PROP_TAG(PT_OBJECT, 32882)
PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_T = PROP_TAG(PT_MV_TSTRING, 32882)
PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL = PROP_TAG(PT_MV_TSTRING, 32883)
PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_A = PROP_TAG(PT_MV_STRING8, 32883)
PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_W = PROP_TAG(PT_MV_UNICODE, 32883)
PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_O = PROP_TAG(PT_OBJECT, 32883)
PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_T = PROP_TAG(PT_MV_TSTRING, 32883)
PR_EMS_AB_DL_MEMBER_RULE = PROP_TAG(PT_MV_BINARY, 32884)
PR_EMS_AB_DOMAIN_DEF_ALT_RECIP = PROP_TAG(PT_TSTRING, 32885)
PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_A = PROP_TAG(PT_STRING8, 32885)
PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_W = PROP_TAG(PT_UNICODE, 32885)
PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_O = PROP_TAG(PT_OBJECT, 32885)
PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_T = PROP_TAG(PT_TSTRING, 32885)
PR_EMS_AB_DOMAIN_NAME = PROP_TAG(PT_TSTRING, 32886)
PR_EMS_AB_DOMAIN_NAME_A = PROP_TAG(PT_STRING8, 32886)
PR_EMS_AB_DOMAIN_NAME_W = PROP_TAG(PT_UNICODE, 32886)
PR_EMS_AB_DSA_SIGNATURE = PROP_TAG(PT_BINARY, 32887)
PR_EMS_AB_DXA_ADMIN_COPY = PROP_TAG(PT_BOOLEAN, 32888)
PR_EMS_AB_DXA_ADMIN_FORWARD = PROP_TAG(PT_BOOLEAN, 32889)
PR_EMS_AB_DXA_ADMIN_UPDATE = PROP_TAG(PT_LONG, 32890)
PR_EMS_AB_DXA_APPEND_REQCN = PROP_TAG(PT_BOOLEAN, 32891)
PR_EMS_AB_DXA_CONF_CONTAINER_LIST = PROP_TAG(PT_MV_TSTRING, 32892)
PR_EMS_AB_DXA_CONF_CONTAINER_LIST_A = PROP_TAG(PT_MV_STRING8, 32892)
PR_EMS_AB_DXA_CONF_CONTAINER_LIST_W = PROP_TAG(PT_MV_UNICODE, 32892)
PR_EMS_AB_DXA_CONF_CONTAINER_LIST_O = PROP_TAG(PT_OBJECT, 32892)
PR_EMS_AB_DXA_CONF_CONTAINER_LIST_T = PROP_TAG(PT_MV_TSTRING, 32892)
PR_EMS_AB_DXA_CONF_REQ_TIME = PROP_TAG(PT_SYSTIME, 32893)
PR_EMS_AB_DXA_CONF_SEQ = PROP_TAG(PT_TSTRING, 32894)
PR_EMS_AB_DXA_CONF_SEQ_A = PROP_TAG(PT_STRING8, 32894)
PR_EMS_AB_DXA_CONF_SEQ_W = PROP_TAG(PT_UNICODE, 32894)
PR_EMS_AB_DXA_CONF_SEQ_USN = PROP_TAG(PT_LONG, 32895)
PR_EMS_AB_DXA_EXCHANGE_OPTIONS = PROP_TAG(PT_LONG, 32896)
PR_EMS_AB_DXA_EXPORT_NOW = PROP_TAG(PT_BOOLEAN, 32897)
PR_EMS_AB_DXA_FLAGS = PROP_TAG(PT_LONG, 32898)
PR_EMS_AB_DXA_IMP_SEQ = PROP_TAG(PT_TSTRING, 32899)
PR_EMS_AB_DXA_IMP_SEQ_A = PROP_TAG(PT_STRING8, 32899)
PR_EMS_AB_DXA_IMP_SEQ_W = PROP_TAG(PT_UNICODE, 32899)
PR_EMS_AB_DXA_IMP_SEQ_TIME = PROP_TAG(PT_SYSTIME, 32900)
PR_EMS_AB_DXA_IMP_SEQ_USN = PROP_TAG(PT_LONG, 32901)
PR_EMS_AB_DXA_IMPORT_NOW = PROP_TAG(PT_BOOLEAN, 32902)
PR_EMS_AB_DXA_IN_TEMPLATE_MAP = PROP_TAG(PT_MV_TSTRING, 32903)
PR_EMS_AB_DXA_IN_TEMPLATE_MAP_A = PROP_TAG(PT_MV_STRING8, 32903)
PR_EMS_AB_DXA_IN_TEMPLATE_MAP_W = PROP_TAG(PT_MV_UNICODE, 32903)
PR_EMS_AB_DXA_LOCAL_ADMIN = PROP_TAG(PT_TSTRING, 32904)
PR_EMS_AB_DXA_LOCAL_ADMIN_A = PROP_TAG(PT_STRING8, 32904)
PR_EMS_AB_DXA_LOCAL_ADMIN_W = PROP_TAG(PT_UNICODE, 32904)
PR_EMS_AB_DXA_LOCAL_ADMIN_O = PROP_TAG(PT_OBJECT, 32904)
PR_EMS_AB_DXA_LOCAL_ADMIN_T = PROP_TAG(PT_TSTRING, 32904)
PR_EMS_AB_DXA_LOGGING_LEVEL = PROP_TAG(PT_LONG, 32905)
PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE = PROP_TAG(PT_TSTRING, 32906)
PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE_A = PROP_TAG(PT_STRING8, 32906)
PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE_W = PROP_TAG(PT_UNICODE, 32906)
PR_EMS_AB_DXA_OUT_TEMPLATE_MAP = PROP_TAG(PT_MV_TSTRING, 32907)
PR_EMS_AB_DXA_OUT_TEMPLATE_MAP_A = PROP_TAG(PT_MV_STRING8, 32907)
PR_EMS_AB_DXA_OUT_TEMPLATE_MAP_W = PROP_TAG(PT_MV_UNICODE, 32907)
PR_EMS_AB_DXA_PASSWORD = PROP_TAG(PT_TSTRING, 32908)
PR_EMS_AB_DXA_PASSWORD_A = PROP_TAG(PT_STRING8, 32908)
PR_EMS_AB_DXA_PASSWORD_W = PROP_TAG(PT_UNICODE, 32908)
PR_EMS_AB_DXA_PREV_EXCHANGE_OPTIONS = PROP_TAG(PT_LONG, 32909)
PR_EMS_AB_DXA_PREV_EXPORT_NATIVE_ONLY = PROP_TAG(PT_BOOLEAN, 32910)
PR_EMS_AB_DXA_PREV_IN_EXCHANGE_SENSITIVITY = PROP_TAG(PT_LONG, 32911)
PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES = PROP_TAG(PT_TSTRING, 32912)
PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_A = PROP_TAG(PT_STRING8, 32912)
PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_W = PROP_TAG(PT_UNICODE, 32912)
PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_O = PROP_TAG(PT_OBJECT, 32912)
PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_T = PROP_TAG(PT_TSTRING, 32912)
PR_EMS_AB_DXA_PREV_REPLICATION_SENSITIVITY = PROP_TAG(PT_LONG, 32913)
PR_EMS_AB_DXA_PREV_TEMPLATE_OPTIONS = PROP_TAG(PT_LONG, 32914)
PR_EMS_AB_DXA_PREV_TYPES = PROP_TAG(PT_LONG, 32915)
PR_EMS_AB_DXA_RECIPIENT_CP = PROP_TAG(PT_TSTRING, 32916)
PR_EMS_AB_DXA_RECIPIENT_CP_A = PROP_TAG(PT_STRING8, 32916)
PR_EMS_AB_DXA_RECIPIENT_CP_W = PROP_TAG(PT_UNICODE, 32916)
PR_EMS_AB_DXA_REMOTE_CLIENT = PROP_TAG(PT_TSTRING, 32917)
PR_EMS_AB_DXA_REMOTE_CLIENT_A = PROP_TAG(PT_STRING8, 32917)
PR_EMS_AB_DXA_REMOTE_CLIENT_W = PROP_TAG(PT_UNICODE, 32917)
PR_EMS_AB_DXA_REMOTE_CLIENT_O = PROP_TAG(PT_OBJECT, 32917)
PR_EMS_AB_DXA_REMOTE_CLIENT_T = PROP_TAG(PT_TSTRING, 32917)
PR_EMS_AB_DXA_REQ_SEQ = PROP_TAG(PT_TSTRING, 32918)
PR_EMS_AB_DXA_REQ_SEQ_A = PROP_TAG(PT_STRING8, 32918)
PR_EMS_AB_DXA_REQ_SEQ_W = PROP_TAG(PT_UNICODE, 32918)
PR_EMS_AB_DXA_REQ_SEQ_TIME = PROP_TAG(PT_SYSTIME, 32919)
PR_EMS_AB_DXA_REQ_SEQ_USN = PROP_TAG(PT_LONG, 32920)
PR_EMS_AB_DXA_REQNAME = PROP_TAG(PT_TSTRING, 32921)
PR_EMS_AB_DXA_REQNAME_A = PROP_TAG(PT_STRING8, 32921)
PR_EMS_AB_DXA_REQNAME_W = PROP_TAG(PT_UNICODE, 32921)
PR_EMS_AB_DXA_SVR_SEQ = PROP_TAG(PT_TSTRING, 32922)
PR_EMS_AB_DXA_SVR_SEQ_A = PROP_TAG(PT_STRING8, 32922)
PR_EMS_AB_DXA_SVR_SEQ_W = PROP_TAG(PT_UNICODE, 32922)
PR_EMS_AB_DXA_SVR_SEQ_TIME = PROP_TAG(PT_SYSTIME, 32923)
PR_EMS_AB_DXA_SVR_SEQ_USN = PROP_TAG(PT_LONG, 32924)
PR_EMS_AB_DXA_TASK = PROP_TAG(PT_LONG, 32925)
PR_EMS_AB_DXA_TEMPLATE_OPTIONS = PROP_TAG(PT_LONG, 32926)
PR_EMS_AB_DXA_TEMPLATE_TIMESTAMP = PROP_TAG(PT_SYSTIME, 32927)
PR_EMS_AB_DXA_TYPES = PROP_TAG(PT_LONG, 32928)
PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST = PROP_TAG(PT_MV_TSTRING, 32929)
PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_A = PROP_TAG(PT_MV_STRING8, 32929)
PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_W = PROP_TAG(PT_MV_UNICODE, 32929)
PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_O = PROP_TAG(PT_OBJECT, 32929)
PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_T = PROP_TAG(PT_MV_TSTRING, 32929)
PR_EMS_AB_ENABLED_PROTOCOLS = PROP_TAG(PT_LONG, 33151)
PR_EMS_AB_ENCAPSULATION_METHOD = PROP_TAG(PT_LONG, 32930)
PR_EMS_AB_ENCRYPT = PROP_TAG(PT_BOOLEAN, 32931)
PR_EMS_AB_ENCRYPT_ALG_LIST_NA = PROP_TAG(PT_MV_TSTRING, 32832)
PR_EMS_AB_ENCRYPT_ALG_LIST_NA_A = PROP_TAG(PT_MV_STRING8, 32832)
PR_EMS_AB_ENCRYPT_ALG_LIST_NA_W = PROP_TAG(PT_MV_UNICODE, 32832)
PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER = PROP_TAG(PT_MV_TSTRING, 32833)
PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER_A = PROP_TAG(PT_MV_STRING8, 32833)
PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER_W = PROP_TAG(PT_MV_UNICODE, 32833)
PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA = PROP_TAG(PT_TSTRING, 32835)
PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA_A = PROP_TAG(PT_STRING8, 32835)
PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA_W = PROP_TAG(PT_UNICODE, 32835)
PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER = PROP_TAG(PT_TSTRING, 32829)
PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER_A = PROP_TAG(PT_STRING8, 32829)
PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER_W = PROP_TAG(PT_UNICODE, 32829)
PR_EMS_AB_EXPAND_DLS_LOCALLY = PROP_TAG(PT_BOOLEAN, 32932)
PR_EMS_AB_EXPIRATION_TIME = PROP_TAG(PT_SYSTIME, 32808)
PR_EMS_AB_EXPORT_CONTAINERS = PROP_TAG(PT_MV_TSTRING, 32933)
PR_EMS_AB_EXPORT_CONTAINERS_A = PROP_TAG(PT_MV_STRING8, 32933)
PR_EMS_AB_EXPORT_CONTAINERS_W = PROP_TAG(PT_MV_UNICODE, 32933)
PR_EMS_AB_EXPORT_CONTAINERS_O = PROP_TAG(PT_OBJECT, 32933)
PR_EMS_AB_EXPORT_CONTAINERS_T = PROP_TAG(PT_MV_TSTRING, 32933)
PR_EMS_AB_EXPORT_CUSTOM_RECIPIENTS = PROP_TAG(PT_BOOLEAN, 32934)
PR_EMS_AB_EXTENDED_CHARS_ALLOWED = PROP_TAG(PT_BOOLEAN, 32935)
PR_EMS_AB_EXTENSION_ATTRIBUTE_1 = PROP_TAG(PT_TSTRING, 32813)
PR_EMS_AB_EXTENSION_ATTRIBUTE_1_A = PROP_TAG(PT_STRING8, 32813)
PR_EMS_AB_EXTENSION_ATTRIBUTE_1_W = PROP_TAG(PT_UNICODE, 32813)
PR_EMS_AB_EXTENSION_ATTRIBUTE_10 = PROP_TAG(PT_TSTRING, 32822)
PR_EMS_AB_EXTENSION_ATTRIBUTE_10_A = PROP_TAG(PT_STRING8, 32822)
PR_EMS_AB_EXTENSION_ATTRIBUTE_10_W = PROP_TAG(PT_UNICODE, 32822)
PR_EMS_AB_EXTENSION_ATTRIBUTE_2 = PROP_TAG(PT_TSTRING, 32814)
PR_EMS_AB_EXTENSION_ATTRIBUTE_2_A = PROP_TAG(PT_STRING8, 32814)
PR_EMS_AB_EXTENSION_ATTRIBUTE_2_W = PROP_TAG(PT_UNICODE, 32814)
PR_EMS_AB_EXTENSION_ATTRIBUTE_3 = PROP_TAG(PT_TSTRING, 32815)
PR_EMS_AB_EXTENSION_ATTRIBUTE_3_A = PROP_TAG(PT_STRING8, 32815)
PR_EMS_AB_EXTENSION_ATTRIBUTE_3_W = PROP_TAG(PT_UNICODE, 32815)
PR_EMS_AB_EXTENSION_ATTRIBUTE_4 = PROP_TAG(PT_TSTRING, 32816)
PR_EMS_AB_EXTENSION_ATTRIBUTE_4_A = PROP_TAG(PT_STRING8, 32816)
PR_EMS_AB_EXTENSION_ATTRIBUTE_4_W = PROP_TAG(PT_UNICODE, 32816)
PR_EMS_AB_EXTENSION_ATTRIBUTE_5 = PROP_TAG(PT_TSTRING, 32817)
PR_EMS_AB_EXTENSION_ATTRIBUTE_5_A = PROP_TAG(PT_STRING8, 32817)
PR_EMS_AB_EXTENSION_ATTRIBUTE_5_W = PROP_TAG(PT_UNICODE, 32817)
PR_EMS_AB_EXTENSION_ATTRIBUTE_6 = PROP_TAG(PT_TSTRING, 32818)
PR_EMS_AB_EXTENSION_ATTRIBUTE_6_A = PROP_TAG(PT_STRING8, 32818)
PR_EMS_AB_EXTENSION_ATTRIBUTE_6_W = PROP_TAG(PT_UNICODE, 32818)
PR_EMS_AB_EXTENSION_ATTRIBUTE_7 = PROP_TAG(PT_TSTRING, 32819)
PR_EMS_AB_EXTENSION_ATTRIBUTE_7_A = PROP_TAG(PT_STRING8, 32819)
PR_EMS_AB_EXTENSION_ATTRIBUTE_7_W = PROP_TAG(PT_UNICODE, 32819)
PR_EMS_AB_EXTENSION_ATTRIBUTE_8 = PROP_TAG(PT_TSTRING, 32820)
PR_EMS_AB_EXTENSION_ATTRIBUTE_8_A = PROP_TAG(PT_STRING8, 32820)
PR_EMS_AB_EXTENSION_ATTRIBUTE_8_W = PROP_TAG(PT_UNICODE, 32820)
PR_EMS_AB_EXTENSION_ATTRIBUTE_9 = PROP_TAG(PT_TSTRING, 32821)
PR_EMS_AB_EXTENSION_ATTRIBUTE_9_A = PROP_TAG(PT_STRING8, 32821)
PR_EMS_AB_EXTENSION_ATTRIBUTE_9_W = PROP_TAG(PT_UNICODE, 32821)
PR_EMS_AB_EXTENSION_DATA = PROP_TAG(PT_MV_BINARY, 32936)
PR_EMS_AB_EXTENSION_NAME = PROP_TAG(PT_MV_TSTRING, 32937)
PR_EMS_AB_EXTENSION_NAME_A = PROP_TAG(PT_MV_STRING8, 32937)
PR_EMS_AB_EXTENSION_NAME_W = PROP_TAG(PT_MV_UNICODE, 32937)
PR_EMS_AB_EXTENSION_NAME_INHERITED = PROP_TAG(PT_MV_TSTRING, 32938)
PR_EMS_AB_EXTENSION_NAME_INHERITED_A = PROP_TAG(PT_MV_STRING8, 32938)
PR_EMS_AB_EXTENSION_NAME_INHERITED_W = PROP_TAG(PT_MV_UNICODE, 32938)
PR_EMS_AB_FACSIMILE_TELEPHONE_NUMBER = PROP_TAG(PT_MV_BINARY, 32939)
PR_EMS_AB_FILE_VERSION = PROP_TAG(PT_BINARY, 32940)
PR_EMS_AB_FILTER_LOCAL_ADDRESSES = PROP_TAG(PT_BOOLEAN, 32941)
PR_EMS_AB_FOLDER_PATHNAME = PROP_TAG(PT_TSTRING, 32772)
PR_EMS_AB_FOLDER_PATHNAME_A = PROP_TAG(PT_STRING8, 32772)
PR_EMS_AB_FOLDER_PATHNAME_W = PROP_TAG(PT_UNICODE, 32772)
PR_EMS_AB_FOLDERS_CONTAINER = PROP_TAG(PT_TSTRING, 32942)
PR_EMS_AB_FOLDERS_CONTAINER_A = PROP_TAG(PT_STRING8, 32942)
PR_EMS_AB_FOLDERS_CONTAINER_W = PROP_TAG(PT_UNICODE, 32942)
PR_EMS_AB_FOLDERS_CONTAINER_O = PROP_TAG(PT_OBJECT, 32942)
PR_EMS_AB_FOLDERS_CONTAINER_T = PROP_TAG(PT_TSTRING, 32942)
PR_EMS_AB_GARBAGE_COLL_PERIOD = PROP_TAG(PT_LONG, 32943)
PR_EMS_AB_GATEWAY_LOCAL_CRED = PROP_TAG(PT_TSTRING, 32944)
PR_EMS_AB_GATEWAY_LOCAL_CRED_A = PROP_TAG(PT_STRING8, 32944)
PR_EMS_AB_GATEWAY_LOCAL_CRED_W = PROP_TAG(PT_UNICODE, 32944)
PR_EMS_AB_GATEWAY_LOCAL_DESIG = PROP_TAG(PT_TSTRING, 32945)
PR_EMS_AB_GATEWAY_LOCAL_DESIG_A = PROP_TAG(PT_STRING8, 32945)
PR_EMS_AB_GATEWAY_LOCAL_DESIG_W = PROP_TAG(PT_UNICODE, 32945)
PR_EMS_AB_GATEWAY_PROXY = PROP_TAG(PT_MV_TSTRING, 32946)
PR_EMS_AB_GATEWAY_PROXY_A = PROP_TAG(PT_MV_STRING8, 32946)
PR_EMS_AB_GATEWAY_PROXY_W = PROP_TAG(PT_MV_UNICODE, 32946)
PR_EMS_AB_GATEWAY_ROUTING_TREE = PROP_TAG(PT_BINARY, 32947)
PR_EMS_AB_GWART_LAST_MODIFIED = PROP_TAG(PT_SYSTIME, 32948)
PR_EMS_AB_HAS_FULL_REPLICA_NCS = PROP_TAG(PT_MV_TSTRING, 32949)
PR_EMS_AB_HAS_FULL_REPLICA_NCS_A = PROP_TAG(PT_MV_STRING8, 32949)
PR_EMS_AB_HAS_FULL_REPLICA_NCS_W = PROP_TAG(PT_MV_UNICODE, 32949)
PR_EMS_AB_HAS_FULL_REPLICA_NCS_O = PROP_TAG(PT_OBJECT, 32949)
PR_EMS_AB_HAS_FULL_REPLICA_NCS_T = PROP_TAG(PT_MV_TSTRING, 32949)
PR_EMS_AB_HAS_MASTER_NCS = PROP_TAG(PT_MV_TSTRING, 32950)
PR_EMS_AB_HAS_MASTER_NCS_A = PROP_TAG(PT_MV_STRING8, 32950)
PR_EMS_AB_HAS_MASTER_NCS_W = PROP_TAG(PT_MV_UNICODE, 32950)
PR_EMS_AB_HAS_MASTER_NCS_O = PROP_TAG(PT_OBJECT, 32950)
PR_EMS_AB_HAS_MASTER_NCS_T = PROP_TAG(PT_MV_TSTRING, 32950)
PR_EMS_AB_HELP_DATA16 = PROP_TAG(PT_BINARY, 32826)
PR_EMS_AB_HELP_DATA32 = PROP_TAG(PT_BINARY, 32784)
PR_EMS_AB_HELP_FILE_NAME = PROP_TAG(PT_TSTRING, 32827)
PR_EMS_AB_HELP_FILE_NAME_A = PROP_TAG(PT_STRING8, 32827)
PR_EMS_AB_HELP_FILE_NAME_W = PROP_TAG(PT_UNICODE, 32827)
PR_EMS_AB_HEURISTICS = PROP_TAG(PT_LONG, 32951)
PR_EMS_AB_HIDE_DL_MEMBERSHIP = PROP_TAG(PT_BOOLEAN, 32952)
PR_EMS_AB_HIDE_FROM_ADDRESS_BOOK = PROP_TAG(PT_BOOLEAN, 32953)
PR_EMS_AB_HOME_MDB = PROP_TAG(PT_TSTRING, 32774)
PR_EMS_AB_HOME_MDB_A = PROP_TAG(PT_STRING8, 32774)
PR_EMS_AB_HOME_MDB_W = PROP_TAG(PT_UNICODE, 32774)
PR_EMS_AB_HOME_MDB_O = PROP_TAG(PT_OBJECT, 32774)
PR_EMS_AB_HOME_MDB_T = PROP_TAG(PT_TSTRING, 32774)
PR_EMS_AB_HOME_MDB_BL = PROP_TAG(PT_MV_TSTRING, 32788)
PR_EMS_AB_HOME_MDB_BL_A = PROP_TAG(PT_MV_STRING8, 32788)
PR_EMS_AB_HOME_MDB_BL_W = PROP_TAG(PT_MV_UNICODE, 32788)
PR_EMS_AB_HOME_MDB_BL_O = PROP_TAG(PT_OBJECT, 32788)
PR_EMS_AB_HOME_MDB_BL_T = PROP_TAG(PT_MV_TSTRING, 32788)
PR_EMS_AB_HOME_MTA = PROP_TAG(PT_TSTRING, 32775)
PR_EMS_AB_HOME_MTA_A = PROP_TAG(PT_STRING8, 32775)
PR_EMS_AB_HOME_MTA_W = PROP_TAG(PT_UNICODE, 32775)
PR_EMS_AB_HOME_MTA_O = PROP_TAG(PT_OBJECT, 32775)
PR_EMS_AB_HOME_MTA_T = PROP_TAG(PT_TSTRING, 32775)
PR_EMS_AB_HOME_PUBLIC_SERVER = PROP_TAG(PT_TSTRING, 32831)
PR_EMS_AB_HOME_PUBLIC_SERVER_A = PROP_TAG(PT_STRING8, 32831)
PR_EMS_AB_HOME_PUBLIC_SERVER_W = PROP_TAG(PT_UNICODE, 32831)
PR_EMS_AB_HOME_PUBLIC_SERVER_O = PROP_TAG(PT_OBJECT, 32831)
PR_EMS_AB_HOME_PUBLIC_SERVER_T = PROP_TAG(PT_TSTRING, 32831)
PR_EMS_AB_IMPORT_CONTAINER = PROP_TAG(PT_TSTRING, 32954)
PR_EMS_AB_IMPORT_CONTAINER_A = PROP_TAG(PT_STRING8, 32954)
PR_EMS_AB_IMPORT_CONTAINER_W = PROP_TAG(PT_UNICODE, 32954)
PR_EMS_AB_IMPORT_CONTAINER_O = PROP_TAG(PT_OBJECT, 32954)
PR_EMS_AB_IMPORT_CONTAINER_T = PROP_TAG(PT_TSTRING, 32954)
PR_EMS_AB_IMPORT_SENSITIVITY = PROP_TAG(PT_LONG, 32955)
PR_EMS_AB_IMPORTED_FROM = PROP_TAG(PT_TSTRING, 32834)
PR_EMS_AB_IMPORTED_FROM_A = PROP_TAG(PT_STRING8, 32834)
PR_EMS_AB_IMPORTED_FROM_W = PROP_TAG(PT_UNICODE, 32834)
PR_EMS_AB_INBOUND_SITES = PROP_TAG(PT_MV_TSTRING, 32956)
PR_EMS_AB_INBOUND_SITES_A = PROP_TAG(PT_MV_STRING8, 32956)
PR_EMS_AB_INBOUND_SITES_W = PROP_TAG(PT_MV_UNICODE, 32956)
PR_EMS_AB_INBOUND_SITES_O = PROP_TAG(PT_OBJECT, 32956)
PR_EMS_AB_INBOUND_SITES_T = PROP_TAG(PT_MV_TSTRING, 32956)
PR_EMS_AB_INSTANCE_TYPE = PROP_TAG(PT_LONG, 32957)
PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER = PROP_TAG(PT_MV_TSTRING, 32958)
PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER_A = PROP_TAG(PT_MV_STRING8, 32958)
PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER_W = PROP_TAG(PT_MV_UNICODE, 32958)
PR_EMS_AB_INVOCATION_ID = PROP_TAG(PT_BINARY, 32959)
PR_EMS_AB_IS_DELETED = PROP_TAG(PT_BOOLEAN, 32960)
PR_EMS_AB_IS_MEMBER_OF_DL = PROP_TAG(PT_OBJECT, 32776)
PR_EMS_AB_IS_MEMBER_OF_DL_A = PROP_TAG(PT_MV_STRING8, 32776)
PR_EMS_AB_IS_MEMBER_OF_DL_W = PROP_TAG(PT_MV_UNICODE, 32776)
PR_EMS_AB_IS_MEMBER_OF_DL_O = PROP_TAG(PT_OBJECT, 32776)
PR_EMS_AB_IS_MEMBER_OF_DL_T = PROP_TAG(PT_MV_TSTRING, 32776)
PR_EMS_AB_IS_SINGLE_VALUED = PROP_TAG(PT_BOOLEAN, 32961)
PR_EMS_AB_KCC_STATUS = PROP_TAG(PT_MV_BINARY, 32962)
PR_EMS_AB_KM_SERVER = PROP_TAG(PT_TSTRING, 32781)
PR_EMS_AB_KM_SERVER_A = PROP_TAG(PT_STRING8, 32781)
PR_EMS_AB_KM_SERVER_W = PROP_TAG(PT_UNICODE, 32781)
PR_EMS_AB_KM_SERVER_O = PROP_TAG(PT_OBJECT, 32781)
PR_EMS_AB_KM_SERVER_T = PROP_TAG(PT_TSTRING, 32781)
PR_EMS_AB_KNOWLEDGE_INFORMATION = PROP_TAG(PT_MV_TSTRING, 32963)
PR_EMS_AB_KNOWLEDGE_INFORMATION_A = PROP_TAG(PT_MV_STRING8, 32963)
PR_EMS_AB_KNOWLEDGE_INFORMATION_W = PROP_TAG(PT_MV_UNICODE, 32963)
PR_EMS_AB_LANGUAGE = PROP_TAG(PT_LONG, 33144)
PR_EMS_AB_LDAP_DISPLAY_NAME = PROP_TAG(PT_MV_TSTRING, 33137)
PR_EMS_AB_LDAP_DISPLAY_NAME_A = PROP_TAG(PT_MV_STRING8, 33137)
PR_EMS_AB_LDAP_DISPLAY_NAME_W = PROP_TAG(PT_MV_UNICODE, 33137)
PR_EMS_AB_LINE_WRAP = PROP_TAG(PT_LONG, 32964)
PR_EMS_AB_LINK_ID = PROP_TAG(PT_LONG, 32965)
PR_EMS_AB_LOCAL_BRIDGE_HEAD = PROP_TAG(PT_TSTRING, 32966)
PR_EMS_AB_LOCAL_BRIDGE_HEAD_A = PROP_TAG(PT_STRING8, 32966)
PR_EMS_AB_LOCAL_BRIDGE_HEAD_W = PROP_TAG(PT_UNICODE, 32966)
PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS = PROP_TAG(PT_TSTRING, 32967)
PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS_A = PROP_TAG(PT_STRING8, 32967)
PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS_W = PROP_TAG(PT_UNICODE, 32967)
PR_EMS_AB_LOCAL_INITIAL_TURN = PROP_TAG(PT_BOOLEAN, 32968)
PR_EMS_AB_LOCAL_SCOPE = PROP_TAG(PT_MV_TSTRING, 32969)
PR_EMS_AB_LOCAL_SCOPE_A = PROP_TAG(PT_MV_STRING8, 32969)
PR_EMS_AB_LOCAL_SCOPE_W = PROP_TAG(PT_MV_UNICODE, 32969)
PR_EMS_AB_LOCAL_SCOPE_O = PROP_TAG(PT_OBJECT, 32969)
PR_EMS_AB_LOCAL_SCOPE_T = PROP_TAG(PT_MV_TSTRING, 32969)
PR_EMS_AB_LOG_FILENAME = PROP_TAG(PT_TSTRING, 32970)
PR_EMS_AB_LOG_FILENAME_A = PROP_TAG(PT_STRING8, 32970)
PR_EMS_AB_LOG_FILENAME_W = PROP_TAG(PT_UNICODE, 32970)
PR_EMS_AB_LOG_ROLLOVER_INTERVAL = PROP_TAG(PT_LONG, 32971)
PR_EMS_AB_MAINTAIN_AUTOREPLY_HISTORY = PROP_TAG(PT_BOOLEAN, 32972)
PR_EMS_AB_MANAGER = PROP_TAG(PT_OBJECT, 32773)
PR_EMS_AB_MANAGER_A = PROP_TAG(PT_STRING8, 32773)
PR_EMS_AB_MANAGER_W = PROP_TAG(PT_UNICODE, 32773)
PR_EMS_AB_MANAGER_O = PROP_TAG(PT_OBJECT, 32773)
PR_EMS_AB_MANAGER_T = PROP_TAG(PT_TSTRING, 32773)
PR_EMS_AB_MAPI_DISPLAY_TYPE = PROP_TAG(PT_LONG, 32973)
PR_EMS_AB_MAPI_ID = PROP_TAG(PT_LONG, 32974)
PR_EMS_AB_MAXIMUM_OBJECT_ID = PROP_TAG(PT_BINARY, 33129)
PR_EMS_AB_MDB_BACKOFF_INTERVAL = PROP_TAG(PT_LONG, 32975)
PR_EMS_AB_MDB_MSG_TIME_OUT_PERIOD = PROP_TAG(PT_LONG, 32976)
PR_EMS_AB_MDB_OVER_QUOTA_LIMIT = PROP_TAG(PT_LONG, 32977)
PR_EMS_AB_MDB_STORAGE_QUOTA = PROP_TAG(PT_LONG, 32978)
PR_EMS_AB_MDB_UNREAD_LIMIT = PROP_TAG(PT_LONG, 32979)
PR_EMS_AB_MDB_USE_DEFAULTS = PROP_TAG(PT_BOOLEAN, 32980)
PR_EMS_AB_MEMBER = PROP_TAG(PT_OBJECT, 32777)
PR_EMS_AB_MEMBER_A = PROP_TAG(PT_MV_STRING8, 32777)
PR_EMS_AB_MEMBER_W = PROP_TAG(PT_MV_UNICODE, 32777)
PR_EMS_AB_MEMBER_O = PROP_TAG(PT_OBJECT, 32777)
PR_EMS_AB_MEMBER_T = PROP_TAG(PT_MV_TSTRING, 32777)
PR_EMS_AB_MESSAGE_TRACKING_ENABLED = PROP_TAG(PT_BOOLEAN, 32981)
PR_EMS_AB_MONITOR_CLOCK = PROP_TAG(PT_BOOLEAN, 32982)
PR_EMS_AB_MONITOR_SERVERS = PROP_TAG(PT_BOOLEAN, 32983)
PR_EMS_AB_MONITOR_SERVICES = PROP_TAG(PT_BOOLEAN, 32984)
PR_EMS_AB_MONITORED_CONFIGURATIONS = PROP_TAG(PT_MV_TSTRING, 32985)
PR_EMS_AB_MONITORED_CONFIGURATIONS_A = PROP_TAG(PT_MV_STRING8, 32985)
PR_EMS_AB_MONITORED_CONFIGURATIONS_W = PROP_TAG(PT_MV_UNICODE, 32985)
PR_EMS_AB_MONITORED_CONFIGURATIONS_O = PROP_TAG(PT_OBJECT, 32985)
PR_EMS_AB_MONITORED_CONFIGURATIONS_T = PROP_TAG(PT_MV_TSTRING, 32985)
PR_EMS_AB_MONITORED_SERVERS = PROP_TAG(PT_MV_TSTRING, 32986)
PR_EMS_AB_MONITORED_SERVERS_A = PROP_TAG(PT_MV_STRING8, 32986)
PR_EMS_AB_MONITORED_SERVERS_W = PROP_TAG(PT_MV_UNICODE, 32986)
PR_EMS_AB_MONITORED_SERVERS_O = PROP_TAG(PT_OBJECT, 32986)
PR_EMS_AB_MONITORED_SERVERS_T = PROP_TAG(PT_MV_TSTRING, 32986)
PR_EMS_AB_MONITORED_SERVICES = PROP_TAG(PT_MV_TSTRING, 32987)
PR_EMS_AB_MONITORED_SERVICES_A = PROP_TAG(PT_MV_STRING8, 32987)
PR_EMS_AB_MONITORED_SERVICES_W = PROP_TAG(PT_MV_UNICODE, 32987)
PR_EMS_AB_MONITORING_ALERT_DELAY = PROP_TAG(PT_LONG, 32988)
PR_EMS_AB_MONITORING_ALERT_UNITS = PROP_TAG(PT_LONG, 32989)
PR_EMS_AB_MONITORING_AVAILABILITY_STYLE = PROP_TAG(PT_LONG, 32990)
PR_EMS_AB_MONITORING_AVAILABILITY_WINDOW = PROP_TAG(PT_BINARY, 32991)
PR_EMS_AB_MONITORING_CACHED_VIA_MAIL = PROP_TAG(PT_MV_TSTRING, 32992)
PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_A = PROP_TAG(PT_MV_STRING8, 32992)
PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_W = PROP_TAG(PT_MV_UNICODE, 32992)
PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_O = PROP_TAG(PT_OBJECT, 32992)
PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_T = PROP_TAG(PT_MV_TSTRING, 32992)
PR_EMS_AB_MONITORING_CACHED_VIA_RPC = PROP_TAG(PT_MV_TSTRING, 32993)
PR_EMS_AB_MONITORING_CACHED_VIA_RPC_A = PROP_TAG(PT_MV_STRING8, 32993)
PR_EMS_AB_MONITORING_CACHED_VIA_RPC_W = PROP_TAG(PT_MV_UNICODE, 32993)
PR_EMS_AB_MONITORING_CACHED_VIA_RPC_O = PROP_TAG(PT_OBJECT, 32993)
PR_EMS_AB_MONITORING_CACHED_VIA_RPC_T = PROP_TAG(PT_MV_TSTRING, 32993)
PR_EMS_AB_MONITORING_ESCALATION_PROCEDURE = PROP_TAG(PT_MV_BINARY, 32994)
PR_EMS_AB_MONITORING_HOTSITE_POLL_INTERVAL = PROP_TAG(PT_LONG, 32995)
PR_EMS_AB_MONITORING_HOTSITE_POLL_UNITS = PROP_TAG(PT_LONG, 32996)
PR_EMS_AB_MONITORING_MAIL_UPDATE_INTERVAL = PROP_TAG(PT_LONG, 32997)
PR_EMS_AB_MONITORING_MAIL_UPDATE_UNITS = PROP_TAG(PT_LONG, 32998)
PR_EMS_AB_MONITORING_NORMAL_POLL_INTERVAL = PROP_TAG(PT_LONG, 32999)
PR_EMS_AB_MONITORING_NORMAL_POLL_UNITS = PROP_TAG(PT_LONG, 33000)
PR_EMS_AB_MONITORING_RECIPIENTS = PROP_TAG(PT_MV_TSTRING, 33001)
PR_EMS_AB_MONITORING_RECIPIENTS_A = PROP_TAG(PT_MV_STRING8, 33001)
PR_EMS_AB_MONITORING_RECIPIENTS_W = PROP_TAG(PT_MV_UNICODE, 33001)
PR_EMS_AB_MONITORING_RECIPIENTS_O = PROP_TAG(PT_OBJECT, 33001)
PR_EMS_AB_MONITORING_RECIPIENTS_T = PROP_TAG(PT_MV_TSTRING, 33001)
PR_EMS_AB_MONITORING_RECIPIENTS_NDR = PROP_TAG(PT_MV_TSTRING, 33002)
PR_EMS_AB_MONITORING_RECIPIENTS_NDR_A = PROP_TAG(PT_MV_STRING8, 33002)
PR_EMS_AB_MONITORING_RECIPIENTS_NDR_W = PROP_TAG(PT_MV_UNICODE, 33002)
PR_EMS_AB_MONITORING_RECIPIENTS_NDR_O = PROP_TAG(PT_OBJECT, 33002)
PR_EMS_AB_MONITORING_RECIPIENTS_NDR_T = PROP_TAG(PT_MV_TSTRING, 33002)
PR_EMS_AB_MONITORING_RPC_UPDATE_INTERVAL = PROP_TAG(PT_LONG, 33003)
PR_EMS_AB_MONITORING_RPC_UPDATE_UNITS = PROP_TAG(PT_LONG, 33004)
PR_EMS_AB_MONITORING_WARNING_DELAY = PROP_TAG(PT_LONG, 33005)
PR_EMS_AB_MONITORING_WARNING_UNITS = PROP_TAG(PT_LONG, 33006)
PR_EMS_AB_MTA_LOCAL_CRED = PROP_TAG(PT_TSTRING, 33007)
PR_EMS_AB_MTA_LOCAL_CRED_A = PROP_TAG(PT_STRING8, 33007)
PR_EMS_AB_MTA_LOCAL_CRED_W = PROP_TAG(PT_UNICODE, 33007)
PR_EMS_AB_MTA_LOCAL_DESIG = PROP_TAG(PT_TSTRING, 33008)
PR_EMS_AB_MTA_LOCAL_DESIG_A = PROP_TAG(PT_STRING8, 33008)
PR_EMS_AB_MTA_LOCAL_DESIG_W = PROP_TAG(PT_UNICODE, 33008)
PR_EMS_AB_N_ADDRESS = PROP_TAG(PT_BINARY, 33009)
PR_EMS_AB_N_ADDRESS_TYPE = PROP_TAG(PT_LONG, 33010)
PR_EMS_AB_NETWORK_ADDRESS = PROP_TAG(PT_MV_TSTRING, 33136)
PR_EMS_AB_NETWORK_ADDRESS_A = PROP_TAG(PT_MV_STRING8, 33136)
PR_EMS_AB_NETWORK_ADDRESS_W = PROP_TAG(PT_MV_UNICODE, 33136)
PR_EMS_AB_NNTP_CHARACTER_SET = PROP_TAG(PT_TSTRING, 33149)
PR_EMS_AB_NNTP_CHARACTER_SET_A = PROP_TAG(PT_STRING8, 33149)
PR_EMS_AB_NNTP_CHARACTER_SET_W = PROP_TAG(PT_UNICODE, 33149)
PR_EMS_AB_NNTP_CONTENT_FORMAT = PROP_TAG(PT_TSTRING, 33142)
PR_EMS_AB_NNTP_CONTENT_FORMAT_A = PROP_TAG(PT_STRING8, 33142)
PR_EMS_AB_NNTP_CONTENT_FORMAT_W = PROP_TAG(PT_UNICODE, 33142)
PR_EMS_AB_NT_MACHINE_NAME = PROP_TAG(PT_TSTRING, 33011)
PR_EMS_AB_NT_MACHINE_NAME_A = PROP_TAG(PT_STRING8, 33011)
PR_EMS_AB_NT_MACHINE_NAME_W = PROP_TAG(PT_UNICODE, 33011)
PR_EMS_AB_NT_SECURITY_DESCRIPTOR = PROP_TAG(PT_BINARY, 32787)
PR_EMS_AB_NUM_OF_OPEN_RETRIES = PROP_TAG(PT_LONG, 33012)
PR_EMS_AB_NUM_OF_TRANSFER_RETRIES = PROP_TAG(PT_LONG, 33013)
PR_EMS_AB_OBJ_DIST_NAME = PROP_TAG(PT_TSTRING, 32828)
PR_EMS_AB_OBJ_DIST_NAME_A = PROP_TAG(PT_STRING8, 32828)
PR_EMS_AB_OBJ_DIST_NAME_W = PROP_TAG(PT_UNICODE, 32828)
PR_EMS_AB_OBJ_DIST_NAME_O = PROP_TAG(PT_OBJECT, 32828)
PR_EMS_AB_OBJ_DIST_NAME_T = PROP_TAG(PT_TSTRING, 32828)
PR_EMS_AB_OBJECT_CLASS_CATEGORY = PROP_TAG(PT_LONG, 33014)
PR_EMS_AB_OBJECT_VERSION = PROP_TAG(PT_LONG, 33015)
PR_EMS_AB_OFF_LINE_AB_CONTAINERS = PROP_TAG(PT_MV_TSTRING, 33016)
PR_EMS_AB_OFF_LINE_AB_CONTAINERS_A = PROP_TAG(PT_MV_STRING8, 33016)
PR_EMS_AB_OFF_LINE_AB_CONTAINERS_W = PROP_TAG(PT_MV_UNICODE, 33016)
PR_EMS_AB_OFF_LINE_AB_CONTAINERS_O = PROP_TAG(PT_OBJECT, 33016)
PR_EMS_AB_OFF_LINE_AB_CONTAINERS_T = PROP_TAG(PT_MV_TSTRING, 33016)
PR_EMS_AB_OFF_LINE_AB_SCHEDULE = PROP_TAG(PT_BINARY, 33017)
PR_EMS_AB_OFF_LINE_AB_SERVER = PROP_TAG(PT_TSTRING, 33018)
PR_EMS_AB_OFF_LINE_AB_SERVER_A = PROP_TAG(PT_STRING8, 33018)
PR_EMS_AB_OFF_LINE_AB_SERVER_W = PROP_TAG(PT_UNICODE, 33018)
PR_EMS_AB_OFF_LINE_AB_SERVER_O = PROP_TAG(PT_OBJECT, 33018)
PR_EMS_AB_OFF_LINE_AB_SERVER_T = PROP_TAG(PT_TSTRING, 33018)
PR_EMS_AB_OFF_LINE_AB_STYLE = PROP_TAG(PT_LONG, 33019)
PR_EMS_AB_OID_TYPE = PROP_TAG(PT_LONG, 33020)
PR_EMS_AB_OM_OBJECT_CLASS = PROP_TAG(PT_BINARY, 33021)
PR_EMS_AB_OM_SYNTAX = PROP_TAG(PT_LONG, 33022)
PR_EMS_AB_OOF_REPLY_TO_ORIGINATOR = PROP_TAG(PT_BOOLEAN, 33023)
PR_EMS_AB_OPEN_RETRY_INTERVAL = PROP_TAG(PT_LONG, 33024)
PR_EMS_AB_ORGANIZATION_NAME = PROP_TAG(PT_MV_TSTRING, 33025)
PR_EMS_AB_ORGANIZATION_NAME_A = PROP_TAG(PT_MV_STRING8, 33025)
PR_EMS_AB_ORGANIZATION_NAME_W = PROP_TAG(PT_MV_UNICODE, 33025)
PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME = PROP_TAG(PT_MV_TSTRING, 33026)
PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME_A = PROP_TAG(PT_MV_STRING8, 33026)
PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME_W = PROP_TAG(PT_MV_UNICODE, 33026)
PR_EMS_AB_ORIGINAL_DISPLAY_TABLE = PROP_TAG(PT_BINARY, 33027)
PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_MSDOS = PROP_TAG(PT_BINARY, 33028)
PR_EMS_AB_OUTBOUND_SITES = PROP_TAG(PT_MV_TSTRING, 33029)
PR_EMS_AB_OUTBOUND_SITES_A = PROP_TAG(PT_MV_STRING8, 33029)
PR_EMS_AB_OUTBOUND_SITES_W = PROP_TAG(PT_MV_UNICODE, 33029)
PR_EMS_AB_OUTBOUND_SITES_O = PROP_TAG(PT_OBJECT, 33029)
PR_EMS_AB_OUTBOUND_SITES_T = PROP_TAG(PT_MV_TSTRING, 33029)
PR_EMS_AB_OWNER = PROP_TAG(PT_TSTRING, 32780)
PR_EMS_AB_OWNER_A = PROP_TAG(PT_STRING8, 32780)
PR_EMS_AB_OWNER_W = PROP_TAG(PT_UNICODE, 32780)
PR_EMS_AB_OWNER_O = PROP_TAG(PT_OBJECT, 32780)
PR_EMS_AB_OWNER_T = PROP_TAG(PT_TSTRING, 32780)
PR_EMS_AB_OWNER_BL = PROP_TAG(PT_TSTRING, 32804)
PR_EMS_AB_OWNER_BL_A = PROP_TAG(PT_STRING8, 32804)
PR_EMS_AB_OWNER_BL_W = PROP_TAG(PT_UNICODE, 32804)
PR_EMS_AB_OWNER_BL_O = PROP_TAG(PT_OBJECT, 32804)
PR_EMS_AB_OWNER_BL_T = PROP_TAG(PT_TSTRING, 32804)
PR_EMS_AB_P_SELECTOR = PROP_TAG(PT_BINARY, 33030)
PR_EMS_AB_P_SELECTOR_INBOUND = PROP_TAG(PT_BINARY, 33031)
PR_EMS_AB_PER_MSG_DIALOG_DISPLAY_TABLE = PROP_TAG(PT_BINARY, 33032)
PR_EMS_AB_PER_RECIP_DIALOG_DISPLAY_TABLE = PROP_TAG(PT_BINARY, 33033)
PR_EMS_AB_PERIOD_REP_SYNC_TIMES = PROP_TAG(PT_BINARY, 33034)
PR_EMS_AB_PERIOD_REPL_STAGGER = PROP_TAG(PT_LONG, 33035)
PR_EMS_AB_PF_CONTACTS = PROP_TAG(PT_MV_TSTRING, 32824)
PR_EMS_AB_PF_CONTACTS_A = PROP_TAG(PT_MV_STRING8, 32824)
PR_EMS_AB_PF_CONTACTS_W = PROP_TAG(PT_MV_UNICODE, 32824)
PR_EMS_AB_PF_CONTACTS_O = PROP_TAG(PT_OBJECT, 32824)
PR_EMS_AB_PF_CONTACTS_T = PROP_TAG(PT_MV_TSTRING, 32824)
PR_EMS_AB_POP_CHARACTER_SET = PROP_TAG(PT_TSTRING, 33145)
PR_EMS_AB_POP_CHARACTER_SET_A = PROP_TAG(PT_STRING8, 33145)
PR_EMS_AB_POP_CHARACTER_SET_W = PROP_TAG(PT_UNICODE, 33145)
PR_EMS_AB_POP_CONTENT_FORMAT = PROP_TAG(PT_TSTRING, 33143)
PR_EMS_AB_POP_CONTENT_FORMAT_A = PROP_TAG(PT_STRING8, 33143)
PR_EMS_AB_POP_CONTENT_FORMAT_W = PROP_TAG(PT_UNICODE, 33143)
PR_EMS_AB_POSTAL_ADDRESS = PROP_TAG(PT_MV_BINARY, 33036)
PR_EMS_AB_PREFERRED_DELIVERY_METHOD = PROP_TAG(PT_MV_LONG, 33037)
PR_EMS_AB_PRMD = PROP_TAG(PT_TSTRING, 33038)
PR_EMS_AB_PRMD_A = PROP_TAG(PT_STRING8, 33038)
PR_EMS_AB_PRMD_W = PROP_TAG(PT_UNICODE, 33038)
PR_EMS_AB_PROXY_ADDRESSES = PROP_TAG(PT_MV_TSTRING, 32783)
PR_EMS_AB_PROXY_ADDRESSES_A = PROP_TAG(PT_MV_STRING8, 32783)
PR_EMS_AB_PROXY_ADDRESSES_W = PROP_TAG(PT_MV_UNICODE, 32783)
PR_EMS_AB_PROXY_GENERATOR_DLL = PROP_TAG(PT_TSTRING, 33039)
PR_EMS_AB_PROXY_GENERATOR_DLL_A = PROP_TAG(PT_STRING8, 33039)
PR_EMS_AB_PROXY_GENERATOR_DLL_W = PROP_TAG(PT_UNICODE, 33039)
PR_EMS_AB_PUBLIC_DELEGATES = PROP_TAG(PT_OBJECT, 32789)
PR_EMS_AB_PUBLIC_DELEGATES_A = PROP_TAG(PT_MV_STRING8, 32789)
PR_EMS_AB_PUBLIC_DELEGATES_W = PROP_TAG(PT_MV_UNICODE, 32789)
PR_EMS_AB_PUBLIC_DELEGATES_O = PROP_TAG(PT_OBJECT, 32789)
PR_EMS_AB_PUBLIC_DELEGATES_T = PROP_TAG(PT_MV_TSTRING, 32789)
PR_EMS_AB_PUBLIC_DELEGATES_BL = PROP_TAG(PT_MV_TSTRING, 33040)
PR_EMS_AB_PUBLIC_DELEGATES_BL_A = PROP_TAG(PT_MV_STRING8, 33040)
PR_EMS_AB_PUBLIC_DELEGATES_BL_W = PROP_TAG(PT_MV_UNICODE, 33040)
PR_EMS_AB_PUBLIC_DELEGATES_BL_O = PROP_TAG(PT_OBJECT, 33040)
PR_EMS_AB_PUBLIC_DELEGATES_BL_T = PROP_TAG(PT_MV_TSTRING, 33040)
PR_EMS_AB_QUOTA_NOTIFICATION_SCHEDULE = PROP_TAG(PT_BINARY, 33041)
PR_EMS_AB_QUOTA_NOTIFICATION_STYLE = PROP_TAG(PT_LONG, 33042)
PR_EMS_AB_RANGE_LOWER = PROP_TAG(PT_LONG, 33043)
PR_EMS_AB_RANGE_UPPER = PROP_TAG(PT_LONG, 33044)
PR_EMS_AB_RAS_CALLBACK_NUMBER = PROP_TAG(PT_TSTRING, 33045)
PR_EMS_AB_RAS_CALLBACK_NUMBER_A = PROP_TAG(PT_STRING8, 33045)
PR_EMS_AB_RAS_CALLBACK_NUMBER_W = PROP_TAG(PT_UNICODE, 33045)
PR_EMS_AB_RAS_PHONE_NUMBER = PROP_TAG(PT_TSTRING, 33046)
PR_EMS_AB_RAS_PHONE_NUMBER_A = PROP_TAG(PT_STRING8, 33046)
PR_EMS_AB_RAS_PHONE_NUMBER_W = PROP_TAG(PT_UNICODE, 33046)
PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME = PROP_TAG(PT_TSTRING, 33047)
PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME_A = PROP_TAG(PT_STRING8, 33047)
PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME_W = PROP_TAG(PT_UNICODE, 33047)
PR_EMS_AB_RAS_REMOTE_SRVR_NAME = PROP_TAG(PT_TSTRING, 33048)
PR_EMS_AB_RAS_REMOTE_SRVR_NAME_A = PROP_TAG(PT_STRING8, 33048)
PR_EMS_AB_RAS_REMOTE_SRVR_NAME_W = PROP_TAG(PT_UNICODE, 33048)
PR_EMS_AB_REGISTERED_ADDRESS = PROP_TAG(PT_MV_BINARY, 33049)
PR_EMS_AB_REMOTE_BRIDGE_HEAD = PROP_TAG(PT_TSTRING, 33050)
PR_EMS_AB_REMOTE_BRIDGE_HEAD_A = PROP_TAG(PT_STRING8, 33050)
PR_EMS_AB_REMOTE_BRIDGE_HEAD_W = PROP_TAG(PT_UNICODE, 33050)
PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS = PROP_TAG(PT_TSTRING, 33051)
PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS_A = PROP_TAG(PT_STRING8, 33051)
PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS_W = PROP_TAG(PT_UNICODE, 33051)
PR_EMS_AB_REMOTE_OUT_BH_SERVER = PROP_TAG(PT_TSTRING, 33052)
PR_EMS_AB_REMOTE_OUT_BH_SERVER_A = PROP_TAG(PT_STRING8, 33052)
PR_EMS_AB_REMOTE_OUT_BH_SERVER_W = PROP_TAG(PT_UNICODE, 33052)
PR_EMS_AB_REMOTE_OUT_BH_SERVER_O = PROP_TAG(PT_OBJECT, 33052)
PR_EMS_AB_REMOTE_OUT_BH_SERVER_T = PROP_TAG(PT_TSTRING, 33052)
PR_EMS_AB_REMOTE_SITE = PROP_TAG(PT_TSTRING, 33053)
PR_EMS_AB_REMOTE_SITE_A = PROP_TAG(PT_STRING8, 33053)
PR_EMS_AB_REMOTE_SITE_W = PROP_TAG(PT_UNICODE, 33053)
PR_EMS_AB_REMOTE_SITE_O = PROP_TAG(PT_OBJECT, 33053)
PR_EMS_AB_REMOTE_SITE_T = PROP_TAG(PT_TSTRING, 33053)
PR_EMS_AB_REPLICATION_MAIL_MSG_SIZE = PROP_TAG(PT_LONG, 33128)
PR_EMS_AB_REPLICATION_SENSITIVITY = PROP_TAG(PT_LONG, 33054)
PR_EMS_AB_REPLICATION_STAGGER = PROP_TAG(PT_LONG, 33055)
PR_EMS_AB_REPORT_TO_ORIGINATOR = PROP_TAG(PT_BOOLEAN, 33056)
PR_EMS_AB_REPORT_TO_OWNER = PROP_TAG(PT_BOOLEAN, 33057)
PR_EMS_AB_REPORTS = PROP_TAG(PT_OBJECT, 32782)
PR_EMS_AB_REPORTS_A = PROP_TAG(PT_MV_STRING8, 32782)
PR_EMS_AB_REPORTS_W = PROP_TAG(PT_MV_UNICODE, 32782)
PR_EMS_AB_REPORTS_O = PROP_TAG(PT_OBJECT, 32782)
PR_EMS_AB_REPORTS_T = PROP_TAG(PT_MV_TSTRING, 32782)
PR_EMS_AB_REQ_SEQ = PROP_TAG(PT_LONG, 33058)
PR_EMS_AB_RESPONSIBLE_LOCAL_DXA = PROP_TAG(PT_TSTRING, 33059)
PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_A = PROP_TAG(PT_STRING8, 33059)
PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_W = PROP_TAG(PT_UNICODE, 33059)
PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_O = PROP_TAG(PT_OBJECT, 33059)
PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_T = PROP_TAG(PT_TSTRING, 33059)
PR_EMS_AB_RID_SERVER = PROP_TAG(PT_TSTRING, 33060)
PR_EMS_AB_RID_SERVER_A = PROP_TAG(PT_STRING8, 33060)
PR_EMS_AB_RID_SERVER_W = PROP_TAG(PT_UNICODE, 33060)
PR_EMS_AB_RID_SERVER_O = PROP_TAG(PT_OBJECT, 33060)
PR_EMS_AB_RID_SERVER_T = PROP_TAG(PT_TSTRING, 33060)
PR_EMS_AB_ROLE_OCCUPANT = PROP_TAG(PT_MV_TSTRING, 33061)
PR_EMS_AB_ROLE_OCCUPANT_A = PROP_TAG(PT_MV_STRING8, 33061)
PR_EMS_AB_ROLE_OCCUPANT_W = PROP_TAG(PT_MV_UNICODE, 33061)
PR_EMS_AB_ROLE_OCCUPANT_O = PROP_TAG(PT_OBJECT, 33061)
PR_EMS_AB_ROLE_OCCUPANT_T = PROP_TAG(PT_MV_TSTRING, 33061)
PR_EMS_AB_ROUTING_LIST = PROP_TAG(PT_MV_TSTRING, 33062)
PR_EMS_AB_ROUTING_LIST_A = PROP_TAG(PT_MV_STRING8, 33062)
PR_EMS_AB_ROUTING_LIST_W = PROP_TAG(PT_MV_UNICODE, 33062)
PR_EMS_AB_RTS_CHECKPOINT_SIZE = PROP_TAG(PT_LONG, 33063)
PR_EMS_AB_RTS_RECOVERY_TIMEOUT = PROP_TAG(PT_LONG, 33064)
PR_EMS_AB_RTS_WINDOW_SIZE = PROP_TAG(PT_LONG, 33065)
PR_EMS_AB_RUNS_ON = PROP_TAG(PT_MV_TSTRING, 33066)
PR_EMS_AB_RUNS_ON_A = PROP_TAG(PT_MV_STRING8, 33066)
PR_EMS_AB_RUNS_ON_W = PROP_TAG(PT_MV_UNICODE, 33066)
PR_EMS_AB_RUNS_ON_O = PROP_TAG(PT_OBJECT, 33066)
PR_EMS_AB_RUNS_ON_T = PROP_TAG(PT_MV_TSTRING, 33066)
PR_EMS_AB_S_SELECTOR = PROP_TAG(PT_BINARY, 33067)
PR_EMS_AB_S_SELECTOR_INBOUND = PROP_TAG(PT_BINARY, 33068)
PR_EMS_AB_SCHEMA_FLAGS = PROP_TAG(PT_LONG, 33139)
PR_EMS_AB_SCHEMA_VERSION = PROP_TAG(PT_MV_LONG, 33148)
PR_EMS_AB_SEARCH_FLAGS = PROP_TAG(PT_LONG, 33069)
PR_EMS_AB_SEARCH_GUIDE = PROP_TAG(PT_MV_BINARY, 33070)
PR_EMS_AB_SECURITY_PROTOCOL = PROP_TAG(PT_MV_BINARY, 32823)
PR_EMS_AB_SEE_ALSO = PROP_TAG(PT_MV_TSTRING, 33071)
PR_EMS_AB_SEE_ALSO_A = PROP_TAG(PT_MV_STRING8, 33071)
PR_EMS_AB_SEE_ALSO_W = PROP_TAG(PT_MV_UNICODE, 33071)
PR_EMS_AB_SEE_ALSO_O = PROP_TAG(PT_OBJECT, 33071)
PR_EMS_AB_SEE_ALSO_T = PROP_TAG(PT_MV_TSTRING, 33071)
PR_EMS_AB_SERIAL_NUMBER = PROP_TAG(PT_MV_TSTRING, 33072)
PR_EMS_AB_SERIAL_NUMBER_A = PROP_TAG(PT_MV_STRING8, 33072)
PR_EMS_AB_SERIAL_NUMBER_W = PROP_TAG(PT_MV_UNICODE, 33072)
PR_EMS_AB_SERVICE_ACTION_FIRST = PROP_TAG(PT_LONG, 33073)
PR_EMS_AB_SERVICE_ACTION_OTHER = PROP_TAG(PT_LONG, 33074)
PR_EMS_AB_SERVICE_ACTION_SECOND = PROP_TAG(PT_LONG, 33075)
PR_EMS_AB_SERVICE_RESTART_DELAY = PROP_TAG(PT_LONG, 33076)
PR_EMS_AB_SERVICE_RESTART_MESSAGE = PROP_TAG(PT_TSTRING, 33077)
PR_EMS_AB_SERVICE_RESTART_MESSAGE_A = PROP_TAG(PT_STRING8, 33077)
PR_EMS_AB_SERVICE_RESTART_MESSAGE_W = PROP_TAG(PT_UNICODE, 33077)
PR_EMS_AB_SESSION_DISCONNECT_TIMER = PROP_TAG(PT_LONG, 33078)
PR_EMS_AB_SITE_AFFINITY = PROP_TAG(PT_MV_TSTRING, 33079)
PR_EMS_AB_SITE_AFFINITY_A = PROP_TAG(PT_MV_STRING8, 33079)
PR_EMS_AB_SITE_AFFINITY_W = PROP_TAG(PT_MV_UNICODE, 33079)
PR_EMS_AB_SITE_FOLDER_GUID = PROP_TAG(PT_BINARY, 33126)
PR_EMS_AB_SITE_FOLDER_SERVER = PROP_TAG(PT_TSTRING, 33127)
PR_EMS_AB_SITE_FOLDER_SERVER_A = PROP_TAG(PT_STRING8, 33127)
PR_EMS_AB_SITE_FOLDER_SERVER_W = PROP_TAG(PT_UNICODE, 33127)
PR_EMS_AB_SITE_FOLDER_SERVER_O = PROP_TAG(PT_OBJECT, 33127)
PR_EMS_AB_SITE_FOLDER_SERVER_T = PROP_TAG(PT_TSTRING, 33127)
PR_EMS_AB_SITE_PROXY_SPACE = PROP_TAG(PT_MV_TSTRING, 33080)
PR_EMS_AB_SITE_PROXY_SPACE_A = PROP_TAG(PT_MV_STRING8, 33080)
PR_EMS_AB_SITE_PROXY_SPACE_W = PROP_TAG(PT_MV_UNICODE, 33080)
PR_EMS_AB_SPACE_LAST_COMPUTED = PROP_TAG(PT_SYSTIME, 33081)
PR_EMS_AB_STREET_ADDRESS = PROP_TAG(PT_TSTRING, 33082)
PR_EMS_AB_STREET_ADDRESS_A = PROP_TAG(PT_STRING8, 33082)
PR_EMS_AB_STREET_ADDRESS_W = PROP_TAG(PT_UNICODE, 33082)
PR_EMS_AB_SUB_REFS = PROP_TAG(PT_MV_TSTRING, 33083)
PR_EMS_AB_SUB_REFS_A = PROP_TAG(PT_MV_STRING8, 33083)
PR_EMS_AB_SUB_REFS_W = PROP_TAG(PT_MV_UNICODE, 33083)
PR_EMS_AB_SUB_REFS_O = PROP_TAG(PT_OBJECT, 33083)
PR_EMS_AB_SUB_REFS_T = PROP_TAG(PT_MV_TSTRING, 33083)
PR_EMS_AB_SUB_SITE = PROP_TAG(PT_TSTRING, 33147)
PR_EMS_AB_SUB_SITE_A = PROP_TAG(PT_STRING8, 33147)
PR_EMS_AB_SUB_SITE_W = PROP_TAG(PT_UNICODE, 33147)
PR_EMS_AB_SUBMISSION_CONT_LENGTH = PROP_TAG(PT_LONG, 33084)
PR_EMS_AB_SUPPORTED_APPLICATION_CONTEXT = PROP_TAG(PT_MV_BINARY, 33085)
PR_EMS_AB_SUPPORTING_STACK = PROP_TAG(PT_MV_TSTRING, 33086)
PR_EMS_AB_SUPPORTING_STACK_A = PROP_TAG(PT_MV_STRING8, 33086)
PR_EMS_AB_SUPPORTING_STACK_W = PROP_TAG(PT_MV_UNICODE, 33086)
PR_EMS_AB_SUPPORTING_STACK_O = PROP_TAG(PT_OBJECT, 33086)
PR_EMS_AB_SUPPORTING_STACK_T = PROP_TAG(PT_MV_TSTRING, 33086)
PR_EMS_AB_SUPPORTING_STACK_BL = PROP_TAG(PT_MV_TSTRING, 33087)
PR_EMS_AB_SUPPORTING_STACK_BL_A = PROP_TAG(PT_MV_STRING8, 33087)
PR_EMS_AB_SUPPORTING_STACK_BL_W = PROP_TAG(PT_MV_UNICODE, 33087)
PR_EMS_AB_SUPPORTING_STACK_BL_O = PROP_TAG(PT_OBJECT, 33087)
PR_EMS_AB_SUPPORTING_STACK_BL_T = PROP_TAG(PT_MV_TSTRING, 33087)
PR_EMS_AB_T_SELECTOR = PROP_TAG(PT_BINARY, 33088)
PR_EMS_AB_T_SELECTOR_INBOUND = PROP_TAG(PT_BINARY, 33089)
PR_EMS_AB_TARGET_ADDRESS = PROP_TAG(PT_TSTRING, 32785)
PR_EMS_AB_TARGET_ADDRESS_A = PROP_TAG(PT_STRING8, 32785)
PR_EMS_AB_TARGET_ADDRESS_W = PROP_TAG(PT_UNICODE, 32785)
PR_EMS_AB_TARGET_MTAS = PROP_TAG(PT_MV_TSTRING, 33090)
PR_EMS_AB_TARGET_MTAS_A = PROP_TAG(PT_MV_STRING8, 33090)
PR_EMS_AB_TARGET_MTAS_W = PROP_TAG(PT_MV_UNICODE, 33090)
PR_EMS_AB_TELEPHONE_NUMBER = PROP_TAG(PT_MV_TSTRING, 32786)
PR_EMS_AB_TELEPHONE_NUMBER_A = PROP_TAG(PT_MV_STRING8, 32786)
PR_EMS_AB_TELEPHONE_NUMBER_W = PROP_TAG(PT_MV_UNICODE, 32786)
PR_EMS_AB_TELETEX_TERMINAL_IDENTIFIER = PROP_TAG(PT_MV_BINARY, 33091)
PR_EMS_AB_TEMP_ASSOC_THRESHOLD = PROP_TAG(PT_LONG, 33092)
PR_EMS_AB_TOMBSTONE_LIFETIME = PROP_TAG(PT_LONG, 33093)
PR_EMS_AB_TRACKING_LOG_PATH_NAME = PROP_TAG(PT_TSTRING, 33094)
PR_EMS_AB_TRACKING_LOG_PATH_NAME_A = PROP_TAG(PT_STRING8, 33094)
PR_EMS_AB_TRACKING_LOG_PATH_NAME_W = PROP_TAG(PT_UNICODE, 33094)
PR_EMS_AB_TRANS_RETRY_MINS = PROP_TAG(PT_LONG, 33095)
PR_EMS_AB_TRANS_TIMEOUT_MINS = PROP_TAG(PT_LONG, 33096)
PR_EMS_AB_TRANSFER_RETRY_INTERVAL = PROP_TAG(PT_LONG, 33097)
PR_EMS_AB_TRANSFER_TIMEOUT_NON_URGENT = PROP_TAG(PT_LONG, 33098)
PR_EMS_AB_TRANSFER_TIMEOUT_NORMAL = PROP_TAG(PT_LONG, 33099)
PR_EMS_AB_TRANSFER_TIMEOUT_URGENT = PROP_TAG(PT_LONG, 33100)
PR_EMS_AB_TRANSLATION_TABLE_USED = PROP_TAG(PT_LONG, 33101)
PR_EMS_AB_TRANSPORT_EXPEDITED_DATA = PROP_TAG(PT_BOOLEAN, 33102)
PR_EMS_AB_TRUST_LEVEL = PROP_TAG(PT_LONG, 33103)
PR_EMS_AB_TURN_REQUEST_THRESHOLD = PROP_TAG(PT_LONG, 33104)
PR_EMS_AB_TWO_WAY_ALTERNATE_FACILITY = PROP_TAG(PT_BOOLEAN, 33105)
PR_EMS_AB_UNAUTH_ORIG_BL = PROP_TAG(PT_MV_TSTRING, 33106)
PR_EMS_AB_UNAUTH_ORIG_BL_A = PROP_TAG(PT_MV_STRING8, 33106)
PR_EMS_AB_UNAUTH_ORIG_BL_W = PROP_TAG(PT_MV_UNICODE, 33106)
PR_EMS_AB_UNAUTH_ORIG_BL_O = PROP_TAG(PT_OBJECT, 33106)
PR_EMS_AB_UNAUTH_ORIG_BL_T = PROP_TAG(PT_MV_TSTRING, 33106)
PR_EMS_AB_USE_SERVER_VALUES = PROP_TAG(PT_BOOLEAN, 33150)
PR_EMS_AB_USER_PASSWORD = PROP_TAG(PT_MV_BINARY, 33107)
PR_EMS_AB_USN_CHANGED = PROP_TAG(PT_LONG, 32809)
PR_EMS_AB_USN_CREATED = PROP_TAG(PT_LONG, 33108)
PR_EMS_AB_USN_DSA_LAST_OBJ_REMOVED = PROP_TAG(PT_LONG, 33109)
PR_EMS_AB_USN_INTERSITE = PROP_TAG(PT_LONG, 33146)
PR_EMS_AB_USN_LAST_OBJ_REM = PROP_TAG(PT_LONG, 33110)
PR_EMS_AB_USN_SOURCE = PROP_TAG(PT_LONG, 33111)
PR_EMS_AB_WWW_HOME_PAGE = PROP_TAG(PT_TSTRING, 33141)
PR_EMS_AB_WWW_HOME_PAGE_A = PROP_TAG(PT_STRING8, 33141)
PR_EMS_AB_WWW_HOME_PAGE_W = PROP_TAG(PT_UNICODE, 33141)
PR_EMS_AB_X121_ADDRESS = PROP_TAG(PT_MV_TSTRING, 33112)
PR_EMS_AB_X121_ADDRESS_A = PROP_TAG(PT_MV_STRING8, 33112)
PR_EMS_AB_X121_ADDRESS_W = PROP_TAG(PT_MV_UNICODE, 33112)
PR_EMS_AB_X25_CALL_USER_DATA_INCOMING = PROP_TAG(PT_BINARY, 33113)
PR_EMS_AB_X25_CALL_USER_DATA_OUTGOING = PROP_TAG(PT_BINARY, 33114)
PR_EMS_AB_X25_FACILITIES_DATA_INCOMING = PROP_TAG(PT_BINARY, 33115)
PR_EMS_AB_X25_FACILITIES_DATA_OUTGOING = PROP_TAG(PT_BINARY, 33116)
PR_EMS_AB_X25_LEASED_LINE_PORT = PROP_TAG(PT_BINARY, 33117)
PR_EMS_AB_X25_LEASED_OR_SWITCHED = PROP_TAG(PT_BOOLEAN, 33118)
PR_EMS_AB_X25_REMOTE_MTA_PHONE = PROP_TAG(PT_TSTRING, 33119)
PR_EMS_AB_X25_REMOTE_MTA_PHONE_A = PROP_TAG(PT_STRING8, 33119)
PR_EMS_AB_X25_REMOTE_MTA_PHONE_W = PROP_TAG(PT_UNICODE, 33119)
PR_EMS_AB_X400_ATTACHMENT_TYPE = PROP_TAG(PT_BINARY, 33120)
PR_EMS_AB_X400_SELECTOR_SYNTAX = PROP_TAG(PT_LONG, 33121)
PR_EMS_AB_X500_ACCESS_CONTROL_LIST = PROP_TAG(PT_BINARY, 33122)
PR_EMS_AB_XMIT_TIMEOUT_NON_URGENT = PROP_TAG(PT_LONG, 33123)
PR_EMS_AB_XMIT_TIMEOUT_NORMAL = PROP_TAG(PT_LONG, 33124)
PR_EMS_AB_XMIT_TIMEOUT_URGENT = PROP_TAG(PT_LONG, 33125)



#
# Generate scripts needed for serious testing!
#
import win32com, win32com.client.makepy
import win32com.test
import pythoncom
import sys, os

genList = [
    ("msword8", "{00020905-0000-0000-C000-000000000046}", 1033, 8, 0),
]

genDir = "Generated4Test"


def GetGenPath():
    import win32api

    return os.path.join(win32api.GetFullPathName(win32com.test.__path__[0]), genDir)


def GenerateFromRegistered(fname, *loadArgs):
    #       tlb = apply(pythoncom.LoadRegTypeLib, loadArgs)
    genPath = GetGenPath()
    try:
        os.stat(genPath)
    except os.error:
        os.mkdir(genPath)
    # Ensure an __init__ exists.
    open(os.path.join(genPath, "__init__.py"), "w").close()
    print(fname, ": generating -", end=" ")
    f = open(os.path.join(genPath, fname + ".py"), "w")
    win32com.client.makepy.GenerateFromTypeLibSpec(
        loadArgs, f, bQuiet=1, bGUIProgress=1
    )
    f.close()
    print("compiling -", end=" ")
    fullModName = "win32com.test.%s.%s" % (genDir, fname)
    exec("import " + fullModName)
    # Inject the generated module as a top level module.
    sys.modules[fname] = sys.modules[fullModName]
    print("done")


def GenerateAll():
    for args in genList:
        try:
            GenerateFromRegistered(*args)
        except KeyboardInterrupt:
            print("** Interrupted ***")
            break
        except pythoncom.com_error:
            print("** Could not generate test code for ", args[0])


def CleanAll():
    print("Cleaning generated test scripts...")
    try:  # Clear exceptions!
        1 / 0
    except:
        pass
    genPath = GetGenPath()
    for args in genList:
        try:
            name = args[0] + ".py"
            os.unlink(os.path.join(genPath, name))
        except os.error as details:
            if type(details) == type(()) and details[0] != 2:
                print("Could not deleted generated", name, details)
        try:
            name = args[0] + ".pyc"
            os.unlink(os.path.join(genPath, name))
        except os.error as details:
            if type(details) == type(()) and details[0] != 2:
                print("Could not deleted generated", name, details)
        try:
            os.unlink(os.path.join(genPath, "__init__.py"))
        except:
            pass
        try:
            os.unlink(os.path.join(genPath, "__init__.pyc"))
        except:
            pass
    try:
        os.rmdir(genPath)
    except os.error as details:
        print("Could not delete test directory -", details)


if __name__ == "__main__":
    GenerateAll()
    CleanAll()



# Classes which describe interfaces.

from win32com.server.exception import Exception
from win32com.server.util import ListEnumeratorGateway
from win32com.axdebug import axdebug
from win32com.axdebug.util import RaiseNotImpl, _wrap
import pythoncom
import win32com.server.connect
import winerror


class EnumDebugCodeContexts(ListEnumeratorGateway):
    """A class to expose a Python sequence as an EnumDebugCodeContexts

    Create an instance of this class passing a sequence (list, tuple, or
    any sequence protocol supporting object) and it will automatically
    support the EnumDebugCodeContexts interface for the object.

    """

    _com_interfaces_ = [axdebug.IID_IEnumDebugCodeContexts]


class EnumDebugStackFrames(ListEnumeratorGateway):
    """A class to expose a Python sequence as an EnumDebugStackFrames

    Create an instance of this class passing a sequence (list, tuple, or
    any sequence protocol supporting object) and it will automatically
    support the EnumDebugStackFrames interface for the object.

    """

    _com_interfaces_ = [axdebug.IID_IEnumDebugStackFrames]


class EnumDebugApplicationNodes(ListEnumeratorGateway):
    """A class to expose a Python sequence as an EnumDebugStackFrames

    Create an instance of this class passing a sequence (list, tuple, or
    any sequence protocol supporting object) and it will automatically
    support the EnumDebugApplicationNodes interface for the object.

    """

    _com_interfaces_ = [axdebug.IID_IEnumDebugApplicationNodes]


class EnumRemoteDebugApplications(ListEnumeratorGateway):
    _com_interfaces_ = [axdebug.IID_IEnumRemoteDebugApplications]


class EnumRemoteDebugApplicationThreads(ListEnumeratorGateway):
    _com_interfaces_ = [axdebug.IID_IEnumRemoteDebugApplicationThreads]


class DebugDocumentInfo:
    _public_methods_ = ["GetName", "GetDocumentClassId"]
    _com_interfaces_ = [axdebug.IID_IDebugDocumentInfo]

    def __init__(self):
        pass

    def GetName(self, dnt):
        """Get the one of the name of the document
        dnt -- int DOCUMENTNAMETYPE
        """
        RaiseNotImpl("GetName")

    def GetDocumentClassId(self):
        """
        Result must be an IID object (or string representing one).
        """
        RaiseNotImpl("GetDocumentClassId")


class DebugDocumentProvider(DebugDocumentInfo):
    _public_methods_ = DebugDocumentInfo._public_methods_ + ["GetDocument"]
    _com_interfaces_ = DebugDocumentInfo._com_interfaces_ + [
        axdebug.IID_IDebugDocumentProvider
    ]

    def GetDocument(self):
        RaiseNotImpl("GetDocument")


class DebugApplicationNode(DebugDocumentProvider):
    """Provides the functionality of IDebugDocumentProvider, plus a context within a project tree."""

    _public_methods_ = (
        """EnumChildren GetParent SetDocumentProvider
                    Close Attach Detach""".split()
        + DebugDocumentProvider._public_methods_
    )
    _com_interfaces_ = [
        axdebug.IID_IDebugDocumentProvider
    ] + DebugDocumentProvider._com_interfaces_

    def __init__(self):
        DebugDocumentProvider.__init__(self)

    def EnumChildren(self):
        # Result is type PyIEnumDebugApplicationNodes
        RaiseNotImpl("EnumChildren")

    def GetParent(self):
        # result is type PyIDebugApplicationNode
        RaiseNotImpl("GetParent")

    def SetDocumentProvider(self, pddp):  # PyIDebugDocumentProvider pddp
        # void result.
        RaiseNotImpl("SetDocumentProvider")

    def Close(self):
        # void result.
        RaiseNotImpl("Close")

    def Attach(self, parent):  # PyIDebugApplicationNode
        # void result.
        RaiseNotImpl("Attach")

    def Detach(self):
        # void result.
        RaiseNotImpl("Detach")


class DebugApplicationNodeEvents:
    """Event interface for DebugApplicationNode object."""

    _public_methods_ = "onAddChild onRemoveChild onDetach".split()
    _com_interfaces_ = [axdebug.IID_IDebugApplicationNodeEvents]

    def __init__(self):
        pass

    def onAddChild(self, child):  # PyIDebugApplicationNode
        # void result.
        RaiseNotImpl("onAddChild")

    def onRemoveChild(self, child):  # PyIDebugApplicationNode
        # void result.
        RaiseNotImpl("onRemoveChild")

    def onDetach(self):
        # void result.
        RaiseNotImpl("onDetach")

    def onAttach(self, parent):  # PyIDebugApplicationNode
        # void result.
        RaiseNotImpl("onAttach")


class DebugDocument(DebugDocumentInfo):
    """The base interface to all debug documents."""

    _public_methods_ = DebugDocumentInfo._public_methods_
    _com_interfaces_ = [axdebug.IID_IDebugDocument] + DebugDocumentInfo._com_interfaces_


class DebugDocumentText(DebugDocument):
    """The interface to a text only debug document."""

    _com_interfaces_ = [axdebug.IID_IDebugDocumentText] + DebugDocument._com_interfaces_
    _public_methods_ = [
        "GetDocumentAttributes",
        "GetSize",
        "GetPositionOfLine",
        "GetLineOfPosition",
        "GetText",
        "GetPositionOfContext",
        "GetContextOfPosition",
    ] + DebugDocument._public_methods_

    def __init__(self):
        pass

    # IDebugDocumentText
    def GetDocumentAttributes(self):
        # Result is int (TEXT_DOC_ATTR)
        RaiseNotImpl("GetDocumentAttributes")

    def GetSize(self):
        # Result is (numLines, numChars)
        RaiseNotImpl("GetSize")

    def GetPositionOfLine(self, cLineNumber):
        # Result is int char position
        RaiseNotImpl("GetPositionOfLine")

    def GetLineOfPosition(self, charPos):
        # Result is int, int (lineNo, offset)
        RaiseNotImpl("GetLineOfPosition")

    def GetText(self, charPos, maxChars, wantAttr):
        """Params
        charPos -- integer
        maxChars -- integer
        wantAttr -- Should the function compute attributes.

        Return value must be (string, attribtues).  attributes may be
        None if(not wantAttr)
        """
        RaiseNotImpl("GetText")

    def GetPositionOfContext(self, debugDocumentContext):
        """Params
        debugDocumentContext -- a PyIDebugDocumentContext object.

        Return value must be (charPos, numChars)
        """
        RaiseNotImpl("GetPositionOfContext")

    def GetContextOfPosition(self, charPos, maxChars):
        """Params are integers.
        Return value must be PyIDebugDocumentContext object
        """
        print(self)
        RaiseNotImpl("GetContextOfPosition")


class DebugDocumentTextExternalAuthor:
    """Allow external editors to edit file-based debugger documents, and to notify the document when the source file has been changed."""

    _public_methods_ = ["GetPathName", "GetFileName", "NotifyChanged"]
    _com_interfaces_ = [axdebug.IID_IDebugDocumentTextExternalAuthor]

    def __init__(self):
        pass

    def GetPathName(self):
        """Return the full path (including file name) to the document's source file.

        Result must be (filename, fIsOriginal), where
        - if fIsOriginalPath is TRUE if the path refers to the original file for the document.
        - if fIsOriginalPath is FALSE if the path refers to a newly created temporary file.

        raise Exception(winerror.E_FAIL) if no source file can be created/determined.
        """
        RaiseNotImpl("GetPathName")

    def GetFileName(self):
        """Return just the name of the document, with no path information.  (Used for "Save As...")

        Result is a string
        """
        RaiseNotImpl("GetFileName")

    def NotifyChanged(self):
        """Notify the host that the document's source file has been saved and
        that its contents should be refreshed.
        """
        RaiseNotImpl("NotifyChanged")


class DebugDocumentTextEvents:
    _public_methods_ = """onDestroy onInsertText onRemoveText
              onReplaceText onUpdateTextAttributes
              onUpdateDocumentAttributes""".split()
    _com_interfaces_ = [axdebug.IID_IDebugDocumentTextEvents]

    def __init__(self):
        pass

    def onDestroy(self):
        # Result is void.
        RaiseNotImpl("onDestroy")

    def onInsertText(self, cCharacterPosition, cNumToInsert):
        # Result is void.
        RaiseNotImpl("onInsertText")

    def onRemoveText(self, cCharacterPosition, cNumToRemove):
        # Result is void.
        RaiseNotImpl("onRemoveText")

    def onReplaceText(self, cCharacterPosition, cNumToReplace):
        # Result is void.
        RaiseNotImpl("onReplaceText")

    def onUpdateTextAttributes(self, cCharacterPosition, cNumToUpdate):
        # Result is void.
        RaiseNotImpl("onUpdateTextAttributes")

    def onUpdateDocumentAttributes(self, textdocattr):  # TEXT_DOC_ATTR
        # Result is void.
        RaiseNotImpl("onUpdateDocumentAttributes")


class DebugDocumentContext:
    _public_methods_ = ["GetDocument", "EnumCodeContexts"]
    _com_interfaces_ = [axdebug.IID_IDebugDocumentContext]

    def __init__(self):
        pass

    def GetDocument(self):
        """Return value must be a PyIDebugDocument object"""
        RaiseNotImpl("GetDocument")

    def EnumCodeContexts(self):
        """Return value must be a PyIEnumDebugCodeContexts object"""
        RaiseNotImpl("EnumCodeContexts")


class DebugCodeContext:
    _public_methods_ = ["GetDocumentContext", "SetBreakPoint"]
    _com_interfaces_ = [axdebug.IID_IDebugCodeContext]

    def __init__(self):
        pass

    def GetDocumentContext(self):
        """Return value must be a PyIDebugDocumentContext object"""
        RaiseNotImpl("GetDocumentContext")

    def SetBreakPoint(self, bps):
        """bps -- an integer with flags."""
        RaiseNotImpl("SetBreakPoint")


class DebugStackFrame:
    """Abstraction representing a logical stack frame on the stack of a thread."""

    _public_methods_ = [
        "GetCodeContext",
        "GetDescriptionString",
        "GetLanguageString",
        "GetThread",
        "GetDebugProperty",
    ]
    _com_interfaces_ = [axdebug.IID_IDebugStackFrame]

    def __init__(self):
        pass

    def GetCodeContext(self):
        """Returns the current code context associated with the stack frame.

        Return value must be a IDebugCodeContext object
        """
        RaiseNotImpl("GetCodeContext")

    def GetDescriptionString(self, fLong):
        """Returns a textual description of the stack frame.

        fLong -- A flag indicating if the long name is requested.
        """
        RaiseNotImpl("GetDescriptionString")

    def GetLanguageString(self):
        """Returns a short or long textual description of the language.

        fLong -- A flag indicating if the long name is requested.
        """
        RaiseNotImpl("GetLanguageString")

    def GetThread(self):
        """Returns the thread associated with this stack frame.

        Result must be a IDebugApplicationThread
        """
        RaiseNotImpl("GetThread")

    def GetDebugProperty(self):
        RaiseNotImpl("GetDebugProperty")


class DebugDocumentHost:
    """The interface from the IDebugDocumentHelper back to
    the smart host or language engine.  This interface
    exposes host specific functionality such as syntax coloring.
    """

    _public_methods_ = [
        "GetDeferredText",
        "GetScriptTextAttributes",
        "OnCreateDocumentContext",
        "GetPathName",
        "GetFileName",
        "NotifyChanged",
    ]
    _com_interfaces_ = [axdebug.IID_IDebugDocumentHost]

    def __init__(self):
        pass

    def GetDeferredText(self, dwTextStartCookie, maxChars, bWantAttr):
        RaiseNotImpl("GetDeferredText")

    def GetScriptTextAttributes(self, codeText, delimterText, flags):
        # Result must be an attribute sequence of same "length" as the code.
        RaiseNotImpl("GetScriptTextAttributes")

    def OnCreateDocumentContext(self):
        # Result must be a PyIUnknown
        RaiseNotImpl("OnCreateDocumentContext")

    def GetPathName(self):
        # Result must be (string, int) where the int is a BOOL
        # - TRUE if the path refers to the original file for the document.
        # - FALSE if the path refers to a newly created temporary file.
        # - raise Exception(scode=E_FAIL) if no source file can be created/determined.
        RaiseNotImpl("GetPathName")

    def GetFileName(self):
        # Result is a string with just the name of the document, no path information.
        RaiseNotImpl("GetFileName")

    def NotifyChanged(self):
        RaiseNotImpl("NotifyChanged")


# Additional gateway related functions.


class DebugDocumentTextConnectServer:
    _public_methods_ = (
        win32com.server.connect.IConnectionPointContainer_methods
        + win32com.server.connect.IConnectionPoint_methods
    )
    _com_interfaces_ = [
        pythoncom.IID_IConnectionPoint,
        pythoncom.IID_IConnectionPointContainer,
    ]
    # IConnectionPoint interfaces
    def __init__(self):
        self.cookieNo = -1
        self.connections = {}

    def EnumConnections(self):
        RaiseNotImpl("EnumConnections")

    def GetConnectionInterface(self):
        RaiseNotImpl("GetConnectionInterface")

    def GetConnectionPointContainer(self):
        return _wrap(self)

    def Advise(self, pUnk):
        # Creates a connection to the client.  Simply allocate a new cookie,
        # find the clients interface, and store it in a dictionary.
        interface = pUnk.QueryInterface(axdebug.IID_IDebugDocumentTextEvents, 1)
        self.cookieNo = self.cookieNo + 1
        self.connections[self.cookieNo] = interface
        return self.cookieNo

    def Unadvise(self, cookie):
        # Destroy a connection - simply delete interface from the map.
        try:
            del self.connections[cookie]
        except KeyError:
            return Exception(scode=winerror.E_UNEXPECTED)

    # IConnectionPointContainer interfaces
    def EnumConnectionPoints(self):
        RaiseNotImpl("EnumConnectionPoints")

    def FindConnectionPoint(self, iid):
        # Find a connection we support.  Only support the single event interface.
        if iid == axdebug.IID_IDebugDocumentTextEvents:
            return _wrap(self)
        raise Exception(scode=winerror.E_NOINTERFACE)  # ??


class RemoteDebugApplicationEvents:
    _public_methods_ = [
        "OnConnectDebugger",
        "OnDisconnectDebugger",
        "OnSetName",
        "OnDebugOutput",
        "OnClose",
        "OnEnterBreakPoint",
        "OnLeaveBreakPoint",
        "OnCreateThread",
        "OnDestroyThread",
        "OnBreakFlagChange",
    ]
    _com_interfaces_ = [axdebug.IID_IRemoteDebugApplicationEvents]

    def OnConnectDebugger(self, appDebugger):
        """appDebugger -- a PyIApplicationDebugger"""
        RaiseNotImpl("OnConnectDebugger")

    def OnDisconnectDebugger(self):
        RaiseNotImpl("OnDisconnectDebugger")

    def OnSetName(self, name):
        RaiseNotImpl("OnSetName")

    def OnDebugOutput(self, string):
        RaiseNotImpl("OnDebugOutput")

    def OnClose(self):
        RaiseNotImpl("OnClose")

    def OnEnterBreakPoint(self, rdat):
        """rdat -- PyIRemoteDebugApplicationThread"""
        RaiseNotImpl("OnEnterBreakPoint")

    def OnLeaveBreakPoint(self, rdat):
        """rdat -- PyIRemoteDebugApplicationThread"""
        RaiseNotImpl("OnLeaveBreakPoint")

    def OnCreateThread(self, rdat):
        """rdat -- PyIRemoteDebugApplicationThread"""
        RaiseNotImpl("OnCreateThread")

    def OnDestroyThread(self, rdat):
        """rdat -- PyIRemoteDebugApplicationThread"""
        RaiseNotImpl("OnDestroyThread")

    def OnBreakFlagChange(self, abf, rdat):
        """abf -- int - one of the axdebug.APPBREAKFLAGS constants
        rdat -- PyIRemoteDebugApplicationThread
        RaiseNotImpl("OnBreakFlagChange")
        """


class DebugExpressionContext:
    _public_methods_ = ["ParseLanguageText", "GetLanguageInfo"]
    _com_interfaces_ = [axdebug.IID_IDebugExpressionContext]

    def __init__(self):
        pass

    def ParseLanguageText(self, code, radix, delim, flags):
        """
        result is IDebugExpression
        """
        RaiseNotImpl("ParseLanguageText")

    def GetLanguageInfo(self):
        """
        result is (string langName, iid langId)
        """
        RaiseNotImpl("GetLanguageInfo")


class DebugExpression:
    _public_methods_ = [
        "Start",
        "Abort",
        "QueryIsComplete",
        "GetResultAsString",
        "GetResultAsDebugProperty",
    ]
    _com_interfaces_ = [axdebug.IID_IDebugExpression]

    def Start(self, callback):
        """
        callback -- an IDebugExpressionCallback

        result - void
        """
        RaiseNotImpl("Start")

    def Abort(self):
        """
        no params
        result -- void
        """
        RaiseNotImpl("Abort")

    def QueryIsComplete(self):
        """
        no params
        result -- void
        """
        RaiseNotImpl("QueryIsComplete")

    def GetResultAsString(self):
        RaiseNotImpl("GetResultAsString")

    def GetResultAsDebugProperty(self):
        RaiseNotImpl("GetResultAsDebugProperty")


class ProvideExpressionContexts:
    _public_methods_ = ["EnumExpressionContexts"]
    _com_interfaces_ = [axdebug.IID_IProvideExpressionContexts]

    def EnumExpressionContexts(self):
        RaiseNotImpl("EnumExpressionContexts")



import win32gui
import win32api
import win32clipboard
import win32con
import time


class ViewerWindow:
    def __init__(self):
        self.hwndNextViewer = None

    def OnPaint(self, hwnd, msg, wp, lp):
        dc, ps = win32gui.BeginPaint(hwnd)
        wndrect = win32gui.GetClientRect(hwnd)
        wndwidth = wndrect[2] - wndrect[0]
        wndheight = wndrect[3] - wndrect[1]
        win32clipboard.OpenClipboard()
        try:
            try:
                hbitmap = win32clipboard.GetClipboardData(win32clipboard.CF_BITMAP)
            except TypeError:
                font = win32gui.LOGFONT()
                font.lfHeight = 15  # int(wndheight/20)
                font.lfWidth = 15  # font.lfHeight
                #            font.lfWeight=150
                hf = win32gui.CreateFontIndirect(font)
                win32gui.SelectObject(dc, hf)
                win32gui.SetBkMode(dc, win32con.TRANSPARENT)
                win32gui.SetTextColor(dc, win32api.RGB(0, 0, 0))
                win32gui.DrawText(
                    dc,
                    "No bitmaps are in the clipboard\n(try pressing the PrtScn button)",
                    -1,
                    (0, 0, wndwidth, wndheight),
                    win32con.DT_CENTER,
                )
            else:
                bminfo = win32gui.GetObject(hbitmap)
                dcDC = win32gui.CreateCompatibleDC(None)
                win32gui.SelectObject(dcDC, hbitmap)
                win32gui.StretchBlt(
                    dc,
                    0,
                    0,
                    wndwidth,
                    wndheight,
                    dcDC,
                    0,
                    0,
                    bminfo.bmWidth,
                    bminfo.bmHeight,
                    win32con.SRCCOPY,
                )
                win32gui.DeleteDC(dcDC)
                win32gui.EndPaint(hwnd, ps)
        finally:
            win32clipboard.CloseClipboard()
        return 0

    def OnDrawClipboard(self, hwnd, msg, wp, lp):
        win32gui.InvalidateRect(hwnd, None, True)

    def OnChangeCBChain(self, hwnd, msg, wp, lp):
        # If the next window is closing, repair the chain.
        if wp == self.hwndNextViewer:
            self.hwndNextViewer = lp
        # Otherwise, pass the message to the next link.
        elif self.hwndNextViewer:
            win32gui.SendMessage(self.hwndNextViewer, msg, wp, lp)

    def OnCreate(self, hwnd, msg, wp, lp):
        self.hwndNextViewer = win32gui.SetClipboardViewer(hwnd)

    def OnClose(self, hwnd, msg, wp, lp):
        win32clipboard.ChangeClipboardChain(hwnd, self.hwndNextViewer)
        win32gui.DestroyWindow(hwnd)
        win32gui.PostQuitMessage(0)

    def go(self):
        wndproc = {
            win32con.WM_PAINT: self.OnPaint,
            win32con.WM_CLOSE: self.OnClose,
            win32con.WM_CREATE: self.OnCreate,
            win32con.WM_DRAWCLIPBOARD: self.OnDrawClipboard,
            win32con.WM_CHANGECBCHAIN: self.OnChangeCBChain,
        }

        wc = win32gui.WNDCLASS()
        wc.lpszClassName = "test_win32clipboard_bmp"
        wc.style = win32con.CS_GLOBALCLASS | win32con.CS_VREDRAW | win32con.CS_HREDRAW
        wc.hbrBackground = win32con.COLOR_WINDOW + 1
        wc.lpfnWndProc = wndproc
        class_atom = win32gui.RegisterClass(wc)
        hwnd = win32gui.CreateWindowEx(
            0,
            class_atom,
            "ClipboardViewer",
            win32con.WS_CAPTION
            | win32con.WS_VISIBLE
            | win32con.WS_THICKFRAME
            | win32con.WS_SYSMENU,
            100,
            100,
            900,
            900,
            0,
            0,
            0,
            None,
        )
        win32clipboard.SetClipboardViewer(hwnd)
        win32gui.PumpMessages()
        win32gui.UnregisterClass(class_atom, None)


if __name__ == "__main__":
    w = ViewerWindow()
    w.go()



# ITransferAdviseSink implementation template

import pythoncom
from win32com.shell import shell, shellcon
from win32com.server.policy import DesignatedWrapPolicy

tsf_flags = list(
    (k, v) for k, v in list(shellcon.__dict__.items()) if k.startswith("TSF_")
)


def decode_flags(flags):
    if flags == 0:
        return "TSF_NORMAL"
    flag_txt = ""
    for k, v in tsf_flags:
        if flags & v:
            if flag_txt:
                flag_txt = flag_txt + "|" + k
            else:
                flag_txt = k
    return flag_txt


TRANSFER_ADVISE_STATES = {}
for k, v in list(shellcon.__dict__.items()):
    if k.startswith("TS_"):
        TRANSFER_ADVISE_STATES[v] = k


def decode_flags(flags):
    if flags == 0:
        return "TSF_NORMAL"
    flag_txt = ""
    for k, v in tsf_flags:
        if flags & v:
            if flag_txt:
                flag_txt = flag_txt + "|" + k
            else:
                flag_txt = k
    return flag_txt


class TransferAdviseSink(DesignatedWrapPolicy):
    _com_interfaces_ = [shell.IID_ITransferAdviseSink]
    _public_methods_ = [
        "UpdateProgress",
        "UpdateTransferState",
        "ConfirmOverwrite",
        "ConfirmEncryptionLoss",
        "FileFailure",
        "SubStreamFailure",
        "PropertyFailure",
    ]

    def __init__(self):
        self._wrap_(self)

    def UpdateProgress(
        self,
        SizeCurrent,
        SizeTotal,
        FilesCurrent,
        FilesTotal,
        FoldersCurrent,
        FoldersTotal,
    ):
        print("UpdateProgress - processed so far:")
        print("\t %s out of %s bytes" % (SizeCurrent, SizeTotal))
        print("\t %s out of %s files" % (FilesCurrent, FilesTotal))
        print("\t %s out of %s folders" % (FoldersCurrent, FoldersTotal))

    def UpdateTransferState(self, State):
        print(
            "Current state: ",
            TRANSFER_ADVISE_STATES.get(State, "??? Unknown state %s ???" % State),
        )

    def ConfirmOverwrite(self, Source, DestParent, Name):
        print(
            "ConfirmOverwrite: ",
            Source.GetDisplayName(shellcon.SHGDN_FORPARSING),
            DestParent.GetDisplayName(shellcon.SHGDN_FORPARSING),
            Name,
        )

    def ConfirmEncryptionLoss(self, Source):
        print(
            "ConfirmEncryptionLoss:", Source.GetDisplayName(shellcon.SHGDN_FORPARSING)
        )

    def FileFailure(self, Item, ItemName, Error):
        print("FileFailure:", Item.GetDisplayName(shellcon.SHGDN_FORPARSING), ItemName)

    def SubStreamFailure(self, Item, StreamName, Error):
        print("SubStreamFailure:\n")

    def PropertyFailure(self, Item, key, Error):
        print("PropertyFailure:\n")


def CreateSink():
    return pythoncom.WrapObject(
        TransferAdviseSink(),
        shell.IID_ITransferAdviseSink,
        shell.IID_ITransferAdviseSink,
    )
